var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { asMaybeUsernameError } from 'edge-core-js';
import * as React from 'react';
import { lstrings } from '../common/locales/strings';
import { TextInputModal } from '../components/modals/TextInputModal';
import { Airship, showError } from '../components/services/AirshipInstance';
/**
 * Ask the user for the username that goes with a recovery key,
 * then launches the questions scene.
 */
export const launchPasswordRecovery = (recoveryKey) => (dispatch, getState, imports) => __awaiter(void 0, void 0, void 0, function* () {
    const { context, onLogEvent = () => { } } = imports;
    function handleSubmit(username) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const questions = yield context.fetchRecovery2Questions(recoveryKey, username);
                dispatch({
                    type: 'NAVIGATE',
                    data: {
                        name: 'recoveryLogin',
                        params: { recoveryKey, userQuestions: questions, username }
                    }
                });
                onLogEvent('Recovery_Username_Success');
                return true;
            }
            catch (error) {
                onLogEvent('Recovery_Username_Failure');
                const usernameError = asMaybeUsernameError(error);
                if (usernameError != null) {
                    showError(lstrings.recovery_by_username_error);
                    return false;
                }
                showError(error);
                return false;
            }
        });
    }
    Airship.show(bridge => (React.createElement(TextInputModal, { bridge: bridge, onSubmit: handleSubmit, title: lstrings.password_recovery, message: lstrings.recover_by_username, inputLabel: lstrings.username, autoCapitalize: "none", autoCorrect: false, autoFocus: true })));
});
