var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as React from 'react';
import { lstrings } from '../common/locales/strings';
import { ButtonsModal } from '../components/modals/ButtonsModal';
import { Airship } from '../components/services/AirshipInstance';
import { getSupportedBiometryType, isTouchEnabled, refreshTouchId } from '../keychain';
import { hasSecurityAlerts } from '../util/hasSecurityAlerts';
import { showNotificationPermissionReminder } from '../util/notificationPermissionReminder';
import { showOtpReminder } from '../util/otpReminder';
/**
 * The user has just logged in, so figure out what do to next.
 */
export const completeLogin = (account) => (dispatch, getState, imports) => __awaiter(void 0, void 0, void 0, function* () {
    // Problem logins:
    const { skipOtpReminder = false, skipSecurityAlerts = false } = imports;
    if (!skipSecurityAlerts && hasSecurityAlerts(account)) {
        dispatch({
            type: 'NAVIGATE',
            data: { name: 'securityAlert', params: { account } }
        });
        return;
    }
    // Recovery logins:
    if (account.recoveryLogin) {
        yield Airship.show(bridge => (React.createElement(ButtonsModal, { bridge: bridge, message: lstrings.recovery_successful, buttons: { ok: { label: lstrings.ok } } })));
        dispatch({
            type: 'NAVIGATE',
            data: { name: 'resecurePassword', params: { account } }
        });
        return;
    }
    // Normal logins:
    if (!skipOtpReminder) {
        yield showOtpReminder(account);
    }
    dispatch(submitLogin(account));
});
/**
 * Everything is done, and we can pass the account to the outside world.
 */
export const submitLogin = (account) => (dispatch, getState, imports) => __awaiter(void 0, void 0, void 0, function* () {
    const { branding, fastLogin = false, onLogEvent, onLogin, onNotificationPermit } = imports;
    account.watch('loggedIn', loggedIn => {
        if (!loggedIn)
            dispatch({ type: 'RESET_APP' });
    });
    if (!fastLogin) {
        yield refreshTouchId(account).catch(e => {
            console.log(e); // Fail quietly
        });
    }
    if (onLogin != null) {
        const touchIdInformation = fastLogin
            ? undefined
            : {
                isTouchSupported: (yield getSupportedBiometryType()) !== false,
                isTouchEnabled: yield isTouchEnabled(account)
            };
        onLogin(account, touchIdInformation);
    }
    if (imports.customPermissionsFunction != null) {
        imports.customPermissionsFunction();
    }
    else if (!fastLogin) {
        yield showNotificationPermissionReminder({
            onLogEvent,
            onNotificationPermit,
            appName: branding === null || branding === void 0 ? void 0 : branding.appName
        }).catch(error => console.log(error));
    }
    // Hide all modals and scenes:
    Airship.clear();
    dispatch({ type: 'NAVIGATE', data: { name: 'loading', params: '' } });
});
