var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import React, { useEffect, useRef, useState } from 'react';
import { useHandler } from '../../hooks/useHandler';
import { showError } from '../services/AirshipInstance';
const DEBOUNCE_DELAY = 500;
export function withExtendedTouchable(WrappedComponent) {
    const ExtendedTouchable = (_a) => {
        var { children, onPress, onLongPress, awaitChild, debounce = false } = _a, restProps = __rest(_a, ["children", "onPress", "onLongPress", "awaitChild", "debounce"]);
        const [isDebouncing, setIsDebouncing] = useState(false);
        const [isAwaiting, setIsAwaiting] = useState(false);
        const timeoutRef = useRef(null);
        useEffect(() => {
            return () => {
                if (timeoutRef.current !== null) {
                    clearTimeout(timeoutRef.current);
                }
            };
        }, []);
        const runPress = (event, pressFn) => {
            if (pressFn == null)
                return;
            if (isDebouncing || isAwaiting)
                return;
            // Set debounce state before we even know if onPress is async or not to
            // guarantee we cannot call onPress twice in quick succession.
            if (debounce)
                setIsDebouncing(true);
            try {
                const result = pressFn(event);
                // Async onPress. Always show busy form until promise resolves
                if (typeof (result === null || result === void 0 ? void 0 : result.then) === 'function') {
                    setIsAwaiting(true);
                    result.catch(e => showError(e)).finally(() => setIsAwaiting(false));
                }
                // Also layer on debounce state toggling if enabled
                if (debounce) {
                    timeoutRef.current = setTimeout(() => setIsDebouncing(false), DEBOUNCE_DELAY);
                }
            }
            catch (err) {
                showError(err);
                // Always reset the busy state no matter what
                setIsDebouncing(false);
                setIsAwaiting(false);
            }
        };
        const handlePress = useHandler((event) => {
            runPress(event, onPress);
        });
        const handleLongPress = useHandler((event) => {
            runPress(event, onLongPress);
        });
        // HACK: Can't get typescript to not complain when defining restProps as T
        return children == null ? null : (React.createElement(WrappedComponent, Object.assign({ disabled: isDebouncing || isAwaiting, onPress: handlePress, onLongPress: handleLongPress }, restProps), isAwaiting && awaitChild != null ? awaitChild : children));
    };
    return ExtendedTouchable;
}
