/**
 * IMPORTANT: Changes in this file MUST be synced between edge-react-gui and
 * edge-login-ui-rn!
 */
import * as React from 'react';
import { useMemo } from 'react';
import { View } from 'react-native';
import { useMarginRemStyle } from '../../hooks/useMarginRemStyle';
export const Space = React.memo((props) => {
    const { children } = props;
    const { aroundRem, horizontalRem, verticalRem, topRem, bottomRem, leftRem, rightRem, expand = false, row = false } = props;
    const { alignBottom, alignLeft, alignRight, alignTop, alignCenter, alignHorizontal, alignVertical } = props;
    const topFill = boolify(alignBottom, alignVertical, alignCenter);
    const bottomFill = boolify(alignTop, alignVertical, alignCenter);
    const leftFill = boolify(alignRight, alignHorizontal, alignCenter);
    const rightFill = boolify(alignLeft, alignHorizontal, alignCenter);
    // Margins:
    const marginRemStyle = useMarginRemStyle({
        bottomRem,
        leftRem,
        rightRem,
        topRem,
        aroundRem,
        horizontalRem,
        verticalRem
    });
    // Direction:
    const flexDirection = row ? 'row' : 'column';
    // Alignment:
    const horizontalAlignment = leftFill && rightFill
        ? 'center'
        : rightFill
            ? 'flex-start'
            : leftFill
                ? 'flex-end'
                : undefined;
    const verticalAlignment = topFill && bottomFill
        ? 'center'
        : bottomFill
            ? 'flex-start'
            : topFill
                ? 'flex-end'
                : undefined;
    const alignItems = row ? verticalAlignment : horizontalAlignment;
    const justifyContent = row
        ? horizontalAlignment !== null && horizontalAlignment !== void 0 ? horizontalAlignment : (expand ? 'space-between' : undefined)
        : verticalAlignment;
    // Flex:
    const alignSelf = expand ? 'stretch' : undefined;
    const flexGrow = expand ? 1 : undefined;
    const spaceStyle = useMemo(() => (Object.assign({ alignSelf,
        alignItems,
        flexGrow,
        flexDirection,
        justifyContent }, marginRemStyle)), [
        alignItems,
        alignSelf,
        flexDirection,
        flexGrow,
        justifyContent,
        marginRemStyle
    ]);
    return React.createElement(View, { style: spaceStyle }, children);
});
const boolify = (...things) => {
    return things.some(thing => {
        return typeof thing === 'boolean' && thing;
    });
};
