import * as React from 'react';
import { initializeLogin } from '../../actions/LoginInitActions';
import { setAppConfig } from '../../common/appConfig';
import { Router } from '../navigation/Router';
import { ReduxStore } from '../services/ReduxStore';
import { changeFont } from '../services/ThemeContext';
import { asExperimentConfig } from './publicTypes';
export function LoginScreen(props) {
    var _a;
    const { context, fastLogin = false, forceLightAccountCreate = false, fontDescription, initialLoginId, onPerfEvent = () => { }, username } = props;
    // Look up the requested user:
    const initialUserInfo = initialLoginId != null
        ? context.localUsers.find(info => info.loginId === initialLoginId)
        : username != null
            ? context.localUsers.find(info => info.username === username)
            : undefined;
    // Update theme fonts if they are different:
    React.useEffect(() => {
        if (fontDescription == null)
            return;
        const { regularFontFamily, headingFontFamily = regularFontFamily } = fontDescription;
        changeFont(regularFontFamily, headingFontFamily);
    }, [fontDescription]);
    setAppConfig(props.appConfig);
    const branding = {
        appId: props.appId,
        appName: props.appName,
        landingSceneText: props.landingScreenText,
        parentButton: props.parentButton,
        primaryLogo: props.primaryLogo,
        primaryLogoCallback: props.primaryLogoCallback
    };
    // Clean to populate with defaults
    const experimentConfig = asExperimentConfig((_a = props.experimentConfig) !== null && _a !== void 0 ? _a : {});
    return (React.createElement(ReduxStore, { imports: {
            accountOptions: props.accountOptions,
            branding,
            context,
            fastLogin: props.fastLogin,
            forceLightAccountCreate,
            initialUserInfo,
            initialRoute: props.initialRoute,
            onComplete: props.onComplete,
            onLogin: props.onLogin,
            onPerfEvent: onPerfEvent,
            onLogEvent: props.onLogEvent,
            onNotificationPermit: props.onNotificationPermit,
            recoveryKey: props.recoveryLogin,
            skipOtpReminder: fastLogin || props.skipOtpReminder,
            skipSecurityAlerts: fastLogin || props.skipSecurityAlerts,
            experimentConfig,
            customPermissionsFunction: props.customPermissionsFunction
        }, initialAction: initializeLogin() },
        React.createElement(Router, { branding: branding })));
}
