var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as React from 'react';
import { Keyboard } from 'react-native';
import { KeyboardAwareScrollView } from 'react-native-keyboard-aware-scroll-view';
import { cacheStyles } from 'react-native-patina';
import { lstrings } from '../../common/locales/strings';
import { useHandler } from '../../hooks/useHandler';
import { useImports } from '../../hooks/useImports';
import { useKeyboardPadding } from '../../hooks/useKeyboardPadding';
import { useDispatch } from '../../types/ReduxTypes';
import { EdgeAnim } from '../common/EdgeAnim';
import { SceneButtons } from '../common/SceneButtons';
import { WarningCard } from '../common/WarningCard';
import { ButtonsModal } from '../modals/ButtonsModal';
import { Airship, showError } from '../services/AirshipInstance';
import { useTheme } from '../services/ThemeContext';
import { EdgeText } from '../themed/EdgeText';
import { FilledTextInput } from '../themed/FilledTextInput';
import { PasswordStatus } from '../themed/PasswordStatus';
import { ThemedScene } from '../themed/ThemedScene';
const ChangePasswordSceneComponent = ({ initPassword, title, onBack, onSkip, onSubmit, mainButtonLabel = lstrings.done }) => {
    const theme = useTheme();
    const styles = getStyles(theme);
    const keyboardPadding = useKeyboardPadding();
    const [passwordReqs, setPasswordReqs] = React.useState({
        minLengthMet: 'unmet',
        hasNumber: 'unmet',
        hasLowercase: 'unmet',
        hasUppercase: 'unmet',
        confirmationMatches: 'unmet'
    });
    const [password, setPassword] = React.useState(initPassword !== null && initPassword !== void 0 ? initPassword : '');
    const [confirmPassword, setConfirmPassword] = React.useState('');
    const [spinner, setSpinner] = React.useState(false);
    const secondInputRef = React.useRef(null);
    // Omit confirmationMatches from next button availability
    const isNextButtonDisabled = Object.entries(passwordReqs)
        .filter(([key]) => key !== 'confirmationMatches') // Exclude confirmationMatches from the check
        .some(([, value]) => value === 'unmet' || value === 'error');
    const handleNext = useHandler(() => __awaiter(void 0, void 0, void 0, function* () {
        const newPasswordReqs = validatePassword(password, confirmPassword, 'error');
        setPasswordReqs(newPasswordReqs);
        if (!Object.entries(newPasswordReqs).some(([, value]) => value === 'unmet' || value === 'error')) {
            setSpinner(true);
            try {
                yield onSubmit(password);
            }
            catch (e) {
                showError(e);
            }
            setSpinner(false);
        }
    }));
    const handleSubmitPasswordField = useHandler(() => {
        var _a;
        (_a = secondInputRef.current) === null || _a === void 0 ? void 0 : _a.focus();
    });
    React.useEffect(() => {
        setPasswordReqs(validatePassword(password, confirmPassword, 'unmet'));
    }, [password, confirmPassword]);
    return (React.createElement(ThemedScene, { onBack: onBack, onSkip: onSkip, title: title },
        React.createElement(KeyboardAwareScrollView, { contentContainerStyle: [styles.container, keyboardPadding], keyboardOpeningTime: 0, keyboardShouldPersistTaps: "handled", enableResetScrollToCoords: false, 
            /**
             * HACK: In iOS:
             * - extraScrollHeight is ONLY respected when tapping to
             * focus a field.
             * - Pressing next does not apply the extraScrollHeight.
             * - Refuses to respect extraScrollHeight while typing, and
             * ALWAYS resets the focus position to make the text field sit just
             * above the keyboard.
             *
             * Android correctly uses this prop to ensure the "next" button is
             * always visible above the keyboard, screen size permitting.
             */
            extraScrollHeight: theme.rem(6), enableAutomaticScroll: true, enableOnAndroid: true },
            React.createElement(EdgeAnim, { enter: { type: 'fadeInUp', distance: 50 }, exit: { type: 'fadeOutDown' } },
                React.createElement(EdgeText, { style: styles.description, numberOfLines: 4 }, lstrings.password_desc)),
            React.createElement(EdgeAnim, { enter: { type: 'fadeInUp', distance: 30 }, exit: { type: 'fadeOutDown' } },
                React.createElement(PasswordStatus, { passwordReqs: passwordReqs })),
            React.createElement(EdgeAnim, { enter: { type: 'fadeInUp', distance: 25 } },
                React.createElement(FilledTextInput, { top: 0.75, horizontal: 0.75, bottom: 0.25, value: password, secureTextEntry: true, returnKeyType: "next", placeholder: lstrings.password, onChangeText: setPassword, onSubmitEditing: handleSubmitPasswordField, clearIcon: true, maxLength: 100 })),
            React.createElement(EdgeAnim, { enter: { type: 'fadeInDown', distance: 25 } },
                React.createElement(FilledTextInput, { ref: secondInputRef, top: 0.25, horizontal: 0.75, value: confirmPassword, secureTextEntry: true, returnKeyType: "done", placeholder: lstrings.confirm_password, onChangeText: setConfirmPassword, onSubmitEditing: handleNext, clearIcon: true, maxLength: 100 })),
            React.createElement(SceneButtons, { primary: {
                    label: mainButtonLabel,
                    disabled: isNextButtonDisabled || confirmPassword === '',
                    onPress: handleNext,
                    spinner
                }, animDistanceStart: 50 }))));
};
const getStyles = cacheStyles((theme) => ({
    container: {
        flexGrow: 1,
        marginHorizontal: theme.rem(0.5),
        alignContent: 'flex-start'
    },
    description: {
        fontSize: theme.rem(0.875),
        marginBottom: theme.rem(0.5)
    },
    actions: {
        flexDirection: 'row',
        justifyContent: 'center',
        marginTop: theme.rem(1)
    }
}));
// The scene for existing users to change their password
export const ChangePasswordScene = (props) => {
    const { route } = props;
    const { account } = route.params;
    const { onComplete = () => { } } = useImports();
    const handleSubmit = useHandler((password) => __awaiter(void 0, void 0, void 0, function* () {
        Keyboard.dismiss();
        yield account.changePassword(password);
        yield Airship.show(bridge => (React.createElement(ButtonsModal, { bridge: bridge, title: lstrings.password_changed, message: lstrings.pwd_change_modal, buttons: { ok: { label: lstrings.ok } } },
            React.createElement(WarningCard, { title: lstrings.warning, header: lstrings.password_change_warning, marginRem: [1, 0.5] }))));
        onComplete();
    }));
    return React.createElement(ChangePasswordSceneComponent, { onSubmit: handleSubmit });
};
// The scene for existing users to recover their password
export const ResecurePasswordScene = (props) => {
    const { route } = props;
    const { account } = route.params;
    const dispatch = useDispatch();
    const handleSkip = useHandler(() => {
        dispatch({
            type: 'NAVIGATE',
            data: { name: 'resecurePin', params: { account } }
        });
    });
    const handleSubmit = useHandler((password) => __awaiter(void 0, void 0, void 0, function* () {
        Keyboard.dismiss();
        yield account.changePassword(password);
        yield Airship.show(bridge => (React.createElement(ButtonsModal, { bridge: bridge, title: lstrings.password_changed, message: lstrings.pwd_change_modal, buttons: { ok: { label: lstrings.ok } } })));
        dispatch({
            type: 'NAVIGATE',
            data: { name: 'resecurePin', params: { account } }
        });
    }));
    return (React.createElement(ChangePasswordSceneComponent, { onSkip: handleSkip, title: lstrings.change_password, onSubmit: handleSubmit }));
};
// The scene for new users to create a password
export const NewAccountPasswordScene = (props) => {
    const { route } = props;
    const { onLogEvent = () => { } } = useImports();
    const dispatch = useDispatch();
    const handleBack = useHandler(() => {
        dispatch({
            type: 'NAVIGATE',
            data: { name: 'newAccountUsername', params: route.params }
        });
    });
    const handleSubmit = useHandler((newPassword) => {
        onLogEvent('Signup_Password_Valid');
        dispatch({
            type: 'NAVIGATE',
            data: {
                name: 'newAccountPin',
                params: Object.assign(Object.assign({}, route.params), { password: newPassword })
            }
        });
    });
    return (React.createElement(ChangePasswordSceneComponent, { onBack: handleBack, onSubmit: handleSubmit, title: lstrings.choose_title_password, mainButtonLabel: lstrings.next_label }));
};
// The scene for light account users to create a password for the upgrade/backup
export const UpgradePasswordScene = (props) => {
    const { route } = props;
    const { onLogEvent = () => { } } = useImports();
    const dispatch = useDispatch();
    const handleBack = useHandler(() => {
        dispatch({
            type: 'NAVIGATE',
            data: { name: 'upgradeUsername', params: route.params }
        });
    });
    const handleSubmit = useHandler((newPassword) => __awaiter(void 0, void 0, void 0, function* () {
        onLogEvent('Backup_Password_Valid');
        dispatch({
            type: 'NAVIGATE',
            data: {
                name: 'upgradeTos',
                params: Object.assign(Object.assign({}, route.params), { password: newPassword })
            }
        });
    }));
    return (React.createElement(ChangePasswordSceneComponent, { onBack: handleBack, onSubmit: handleSubmit, title: lstrings.choose_title_password, mainButtonLabel: lstrings.next_label }));
};
const validatePassword = (password, confirmPassword, failStatus) => {
    return {
        minLengthMet: password.length >= 10 ? 'met' : failStatus,
        hasNumber: /[0-9]/.test(password) ? 'met' : failStatus,
        hasLowercase: /[a-z]/.test(password) ? 'met' : failStatus,
        hasUppercase: /[A-Z]/.test(password) ? 'met' : failStatus,
        confirmationMatches: confirmPassword !== '' && confirmPassword === password
            ? 'met'
            : failStatus
    };
};
