var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as React from 'react';
import { Keyboard, ScrollView } from 'react-native';
import { cacheStyles } from 'react-native-patina';
import { completeLogin, submitLogin } from '../../actions/LoginCompleteActions';
import { maybeRouteComplete } from '../../actions/LoginInitActions';
import { lstrings } from '../../common/locales/strings';
import { useCreateAccountHandler } from '../../hooks/useCreateAccount';
import { useHandler } from '../../hooks/useHandler';
import { useImports } from '../../hooks/useImports';
import { useScrollToEnd } from '../../hooks/useScrollToEnd';
import { useDispatch } from '../../types/ReduxTypes';
import { EdgeAnim } from '../common/EdgeAnim';
import { SceneButtons } from '../common/SceneButtons';
import { ButtonsModal } from '../modals/ButtonsModal';
import { Airship, showError } from '../services/AirshipInstance';
import { useTheme } from '../services/ThemeContext';
import { DigitInput, MAX_PIN_LENGTH } from '../themed/DigitInput';
import { EdgeText } from '../themed/EdgeText';
import { ThemedScene } from '../themed/ThemedScene';
const ChangePinSceneComponent = ({ body = lstrings.pin_desc, initPin, title, onBack, onSkip, onSubmit, mainButtonLabel = lstrings.done }) => {
    const theme = useTheme();
    const styles = getStyles(theme);
    const [pin, setPin] = React.useState(initPin !== null && initPin !== void 0 ? initPin : '');
    const isValidPin = pin.length === MAX_PIN_LENGTH;
    const handlePress = useHandler(() => (isValidPin ? onSubmit(pin) : undefined));
    const handleChangePin = useHandler((newPin) => {
        // Change pin only when input are numbers
        if ((/^\d+$/.test(newPin) || newPin.length === 0) && newPin.length <= 4) {
            setPin(newPin);
            if (newPin.length === 4) {
                Keyboard.dismiss();
            }
        }
    });
    const scrollViewRef = useScrollToEnd(isValidPin);
    return (React.createElement(ThemedScene, { onBack: onBack, onSkip: onSkip, title: title },
        React.createElement(ScrollView, { ref: scrollViewRef, contentContainerStyle: styles.content, keyboardShouldPersistTaps: "handled" },
            React.createElement(EdgeAnim, { enter: { type: 'fadeInUp' } },
                React.createElement(EdgeText, { style: styles.description, numberOfLines: 0 }, body)),
            React.createElement(EdgeAnim, { enter: { type: 'fadeInDown' } },
                React.createElement(DigitInput, { pin: pin, testID: "pinInput", onChangePin: handleChangePin }))),
        React.createElement(SceneButtons, { absolute: true, primary: {
                label: mainButtonLabel,
                onPress: handlePress,
                disabled: !isValidPin
            }, animDistanceStart: 50 })));
};
const getStyles = cacheStyles((theme) => ({
    content: {
        flexGrow: 1,
        marginHorizontal: theme.rem(0.5)
    },
    description: {
        fontSize: theme.rem(0.875),
        marginBottom: theme.rem(2)
    }
}));
// The scene for existing users to change their PIN
export const ChangePinScene = (props) => {
    const { route } = props;
    const { account } = route.params;
    const { onComplete = () => { } } = useImports();
    const handleSubmit = useHandler((pin) => __awaiter(void 0, void 0, void 0, function* () {
        Keyboard.dismiss();
        try {
            yield account.changePin({ pin });
            yield Airship.show(bridge => (React.createElement(ButtonsModal, { bridge: bridge, title: lstrings.pin_changed, message: lstrings.pin_successfully_changed, buttons: { ok: { label: lstrings.ok } } })));
            onComplete();
        }
        catch (e) {
            showError(e);
        }
    }));
    return React.createElement(ChangePinSceneComponent, { onSubmit: handleSubmit });
};
// The scene for new users to recover their PIN
export const ResecurePinScene = (props) => {
    const { route } = props;
    const { account } = route.params;
    const { onComplete = () => { }, onLogin } = useImports();
    const dispatch = useDispatch();
    const handleComplete = () => {
        if (onLogin != null)
            dispatch(submitLogin(account));
        else
            onComplete();
    };
    const handleSubmit = useHandler((pin) => __awaiter(void 0, void 0, void 0, function* () {
        Keyboard.dismiss();
        try {
            yield account.changePin({ pin });
            yield Airship.show(bridge => (React.createElement(ButtonsModal, { bridge: bridge, title: lstrings.pin_changed, message: lstrings.pin_successfully_changed, buttons: { ok: { label: lstrings.ok } } })));
            handleComplete();
        }
        catch (e) {
            showError(e);
        }
    }));
    return (React.createElement(ChangePinSceneComponent, { onSkip: handleComplete, title: lstrings.change_pin, onSubmit: handleSubmit }));
};
// The scene for new users to set their PIN
export const NewAccountPinScene = (props) => {
    const { route } = props;
    const { username, password } = route.params;
    const dispatch = useDispatch();
    const { onLogEvent = () => { } } = useImports();
    const lightAccount = username == null;
    const handleBack = useHandler(() => {
        dispatch(lightAccount
            ? maybeRouteComplete({
                type: 'NAVIGATE',
                data: {
                    name: 'passwordLogin',
                    params: { username: '' }
                }
            })
            : maybeRouteComplete({
                type: 'NAVIGATE',
                data: {
                    name: 'newAccountPassword',
                    params: route.params
                }
            }));
    });
    const handleCreateAccount = useCreateAccountHandler();
    const handleSubmit = useHandler((newPin) => __awaiter(void 0, void 0, void 0, function* () {
        onLogEvent('Signup_PIN_Valid');
        if (lightAccount) {
            let errorText;
            try {
                dispatch({
                    type: 'NAVIGATE',
                    data: {
                        name: 'newAccountWait',
                        params: {
                            title: lstrings.great_job,
                            message: lstrings.hang_tight + '\n' + lstrings.secure_account
                        }
                    }
                });
                const account = yield handleCreateAccount({
                    password,
                    pin: newPin,
                    username
                });
                dispatch(completeLogin(account));
            }
            catch (error) {
                showError(error);
                errorText = String(error);
                dispatch({
                    type: 'NAVIGATE',
                    data: {
                        name: 'newAccountPin',
                        params: route.params
                    }
                });
            }
            onLogEvent('Signup_Create_Light_Account', { error: errorText });
        }
        else {
            dispatch({
                type: 'NAVIGATE',
                data: {
                    name: 'newAccountTos',
                    params: Object.assign(Object.assign({}, route.params), { pin: newPin })
                }
            });
        }
    }));
    return (React.createElement(ChangePinSceneComponent, { body: lightAccount ? lstrings.pin_desc_alt : undefined, title: lstrings.choose_title_pin, onBack: handleBack, onSubmit: handleSubmit, mainButtonLabel: lstrings.next_label }));
};
