var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { asMaybeChallengeError, asMaybeOtpError } from 'edge-core-js';
import * as React from 'react';
import FontAwesome from 'react-native-vector-icons/FontAwesome';
import { sprintf } from 'sprintf-js';
import { completeLogin } from '../../actions/LoginCompleteActions';
import { lstrings } from '../../common/locales/strings';
import { useHandler } from '../../hooks/useHandler';
import { useImports } from '../../hooks/useImports';
import { useDispatch } from '../../types/ReduxTypes';
import { attemptLogin } from '../../util/loginAttempt';
import { makePeriodicTask } from '../../util/periodicTask';
import { toLocalTime } from '../../util/utils';
import { ChallengeModal } from '../modals/ChallengeModal';
import { showResetModal } from '../modals/OtpResetModal';
import { QrCodeModal } from '../modals/QrCodeModal';
import { TextInputModal } from '../modals/TextInputModal';
import { Airship, showError, showToast } from '../services/AirshipInstance';
import { DividerWithText } from '../themed/DividerWithText';
import { IconHeaderRow } from '../themed/IconHeaderRow';
import { LinkRow } from '../themed/LinkRow';
import { ThemedScene } from '../themed/ThemedScene';
import { MessageText, Warning } from '../themed/ThemedText';
export function OtpErrorScene(props) {
    var _a;
    const { route } = props;
    const { otpAttempt, otpError } = route.params;
    const { resetToken, voucherId } = otpError;
    const { accountOptions, context, onPerfEvent } = useImports();
    const dispatch = useDispatch();
    const [otpResetDate, setOtpResetDate] = React.useState(otpError.resetDate);
    const inModal = React.useRef(false);
    //
    // Background refresh loop
    //
    React.useEffect(() => {
        const checkVoucher = makePeriodicTask(() => __awaiter(this, void 0, void 0, function* () {
            try {
                if (inModal.current)
                    return;
                if (voucherId == null)
                    return;
                // Is our voucher pending?
                const messages = yield context.fetchLoginMessages();
                for (const message of messages) {
                    const { pendingVouchers } = message;
                    for (const voucher of pendingVouchers) {
                        if (voucher.voucherId === voucherId)
                            return;
                    }
                }
                showToast(lstrings.otp_scene_retrying);
                const account = yield attemptLogin(context, otpAttempt, accountOptions, onPerfEvent);
                dispatch(completeLogin(account));
            }
            catch (error) {
                const otpError = asMaybeOtpError(error);
                if (otpError != null) {
                    dispatch({
                        type: 'NAVIGATE',
                        data: { name: 'otpError', params: { otpAttempt, otpError } }
                    });
                    setOtpResetDate(otpError.resetDate);
                    return;
                }
                showError(error);
            }
        }), 5000);
        checkVoucher.start();
        return () => checkVoucher.stop();
    }, [accountOptions, context, dispatch, onPerfEvent, otpAttempt, voucherId]);
    //
    // Handlers
    //
    const handleBack = useHandler(() => {
        dispatch({
            type: 'NAVIGATE',
            data: {
                name: 'passwordLogin',
                params: { username: otpAttempt.username }
            }
        });
    });
    const handleBackupModal = useHandler(() => {
        inModal.current = true;
        function handleSubmit(otpKey) {
            return __awaiter(this, void 0, void 0, function* () {
                try {
                    const account = yield attemptLogin(context, otpAttempt, Object.assign(Object.assign({}, accountOptions), { otpKey }), onPerfEvent);
                    dispatch(completeLogin(account));
                    return true;
                }
                catch (error) {
                    // Translate known errors:
                    const otpError = asMaybeOtpError(error);
                    if (otpError != null) {
                        dispatch({
                            type: 'NAVIGATE',
                            data: { name: 'otpError', params: { otpAttempt, otpError } }
                        });
                        setOtpResetDate(otpError.resetDate);
                        return lstrings.backup_key_incorrect;
                    }
                    if (error instanceof Error &&
                        error.message === 'Unexpected end of data') {
                        return lstrings.backup_key_incorrect;
                    }
                    const challengeError = asMaybeChallengeError === null || asMaybeChallengeError === void 0 ? void 0 : asMaybeChallengeError(error);
                    if (challengeError != null) {
                        const result = yield Airship.show(bridge => (React.createElement(ChallengeModal, { bridge: bridge, challengeError: challengeError })));
                        if (result !== true)
                            return lstrings.failed_captcha_error;
                        const account = yield attemptLogin(context, otpAttempt, Object.assign(Object.assign({}, accountOptions), { otpKey, challengeId: challengeError.challengeId }), onPerfEvent);
                        dispatch(completeLogin(account));
                        return true;
                    }
                    showError(error);
                    return false;
                }
            });
        }
        Airship.show(bridge => (React.createElement(TextInputModal, { bridge: bridge, autoCapitalize: "characters", inputLabel: lstrings.backup_key_label, message: lstrings.otp_instructions, returnKeyType: "done", submitLabel: lstrings.submit, title: lstrings.otp_backup_code_modal_title, onSubmit: handleSubmit })))
            .catch(error => showError(error))
            .finally(() => (inModal.current = false));
    });
    const handleQrModal = useHandler(() => __awaiter(this, void 0, void 0, function* () {
        inModal.current = true;
        const account = yield Airship.show(bridge => (React.createElement(QrCodeModal, { bridge: bridge, accountOptions: accountOptions, context: context })));
        if (account != null)
            yield dispatch(completeLogin(account));
        inModal.current = false;
    }));
    const handleResetModal = useHandler(() => __awaiter(this, void 0, void 0, function* () {
        inModal.current = true;
        function handleSubmit() {
            return __awaiter(this, void 0, void 0, function* () {
                if (resetToken == null) {
                    throw new Error('No OTP reset token');
                }
                const date = yield context.requestOtpReset(otpAttempt.username, resetToken);
                setOtpResetDate(date);
            });
        }
        yield showResetModal(handleSubmit);
        inModal.current = false;
    }));
    //
    // Render
    //
    const isIp = otpError.reason === 'ip';
    // Find the automatic login date:
    const date = (_a = otpError.voucherActivates) !== null && _a !== void 0 ? _a : otpResetDate;
    return (React.createElement(ThemedScene, { onBack: handleBack, title: isIp ? lstrings.otp_header_ip : lstrings.otp_header },
        React.createElement(IconHeaderRow, { renderIcon: theme => (React.createElement(Warning, null,
                React.createElement(FontAwesome, { name: "exclamation-triangle", size: theme.rem(2.5) }))) },
            React.createElement(MessageText, null,
                React.createElement(Warning, null, isIp
                    ? lstrings.otp_scene_header_ip
                    : lstrings.otp_scene_header_2fa))),
        React.createElement(DividerWithText, { label: lstrings.to_fix }),
        React.createElement(MessageText, null, lstrings.otp_scene_approve),
        React.createElement(DividerWithText, null),
        React.createElement(LinkRow, { label: lstrings.otp_scene_qr, onPress: handleQrModal }),
        isIp ? null : (React.createElement(React.Fragment, null,
            React.createElement(DividerWithText, null),
            React.createElement(LinkRow, { label: lstrings.otp_backup_code_modal_title, onPress: handleBackupModal }))),
        date == null ? null : (React.createElement(React.Fragment, null,
            React.createElement(DividerWithText, null),
            React.createElement(MessageText, null, sprintf(lstrings.otp_scene_wait, toLocalTime(date))))),
        resetToken == null || date != null ? null : (React.createElement(React.Fragment, null,
            React.createElement(DividerWithText, null),
            React.createElement(LinkRow, { label: lstrings.disable_otp_button_two, onPress: handleResetModal })))));
}
