var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as React from 'react';
import { useMemo, useState } from 'react';
import { Keyboard, ScrollView, Text, View } from 'react-native';
import { cacheStyles } from 'react-native-patina';
import { sprintf } from 'sprintf-js';
import { sendRecoveryEmail, shareRecovery } from '../../../actions/PasswordRecoveryActions';
import { lstrings } from '../../../common/locales/strings';
import { questionsList } from '../../../constants/recoveryQuestions';
import { useImports } from '../../../hooks/useImports';
import { useScrollToEnd } from '../../../hooks/useScrollToEnd';
import { validateEmail } from '../../../util/utils';
import { EdgeTouchableOpacity } from '../../common/EdgeTouchableOpacity';
import { Tile } from '../../common/Tile';
import { WarningCard } from '../../common/WarningCard';
import { ButtonsModal } from '../../modals/ButtonsModal';
import { DateModal } from '../../modals/DateModal';
import { QuestionListModal } from '../../modals/QuestionListModal';
import { TextInputModal } from '../../modals/TextInputModal';
import { Airship, showError } from '../../services/AirshipInstance';
import { useTheme } from '../../services/ThemeContext';
import { MainButton } from '../../themed/MainButton';
import { ModalMessage } from '../../themed/ModalParts';
import { ThemedScene } from '../../themed/ThemedScene';
const NUM_QUESTIONS = 2;
export const ChangeRecoveryScene = (props) => {
    const { branding, route } = props;
    const { account, userQuestions } = route.params;
    const theme = useTheme();
    const styles = getStyles(theme);
    const { onComplete = () => { } } = useImports();
    const questionPrompt = lstrings.choose_recovery_question;
    const answerPrompt = lstrings.your_answer_label;
    const [recoveryLocked, setRecoveryLocked] = useState(userQuestions.length > 0);
    // userQuestions is an array of null's when no recovery exists
    const [questions, setQuestions] = useState([
        ...userQuestions
    ]);
    const [answers, setAnswers] = useState([]);
    const confirmButtonsEnabled = !answers.some(a => a == null) &&
        questions.length === NUM_QUESTIONS &&
        answers.length === questions.length;
    const scrollViewRef = useScrollToEnd(confirmButtonsEnabled);
    const showCaseSensitivityWarning = useMemo(() => questions.some(q => !(q === null || q === void 0 ? void 0 : q.startsWith('date:'))), [questions]);
    const extractQuestion = (recoveryQuestion) => {
        return recoveryQuestion === null || recoveryQuestion === void 0 ? void 0 : recoveryQuestion.split(':').slice(-1)[0];
    };
    const stringPredicate = (s) => typeof s === 'string';
    const handleQuestion = (index) => () => __awaiter(void 0, void 0, void 0, function* () {
        const question = questions[index];
        const items = questionsList.map(extractQuestion).filter(stringPredicate);
        const questionText = yield Airship.show(bridge => (React.createElement(QuestionListModal, { bridge: bridge, title: sprintf(lstrings.recovery_question, index + 1), items: items.filter(item => 
            // Display the currently selected question and the unselected questions
            (question === null || question === void 0 ? void 0 : question.endsWith(item)) || !questions.some(q => q === null || q === void 0 ? void 0 : q.endsWith(item))), selected: extractQuestion(question) })));
        if (questionText == null)
            return;
        const questionIndex = items.indexOf(questionText);
        questions[index] = questionsList[questionIndex];
        setQuestions([...questions]);
        // Reset this Q/A pair's answer
        if (answers[index] !== null) {
            answers[index] = null;
            setAnswers([...answers]);
        }
    });
    const handleAnswer = (index) => () => {
        const question = questions[index];
        if (question == null)
            return;
        if (question.startsWith('date:')) {
            showDatePickerModal(index);
        }
        else if (question.startsWith('text:')) {
            showTextInputModal(index);
        }
        else {
            // Legacy Questions
            showTextInputModal(index);
        }
    };
    const showDatePickerModal = (index) => __awaiter(void 0, void 0, void 0, function* () {
        const now = new Date();
        yield Airship.show(bridge => (React.createElement(DateModal, { bridge: bridge, initialValue: now })))
            .then(answer => {
            const date = answer.toISOString().split('T')[0];
            answers[index] = date;
            setAnswers([...answers]);
        })
            .catch((error) => {
            showError(error);
        });
    });
    const showTextInputModal = (index) => __awaiter(void 0, void 0, void 0, function* () {
        const question = questions[index];
        if (question == null)
            return;
        let minLengthString = question.split(':')[1];
        if (minLengthString == null || minLengthString.trim() === '')
            minLengthString = '0';
        const minLength = parseInt(minLengthString);
        const validateAnswer = (answer) => __awaiter(void 0, void 0, void 0, function* () {
            return answer.length >= minLength
                ? true
                : sprintf(lstrings.min_length_error, minLength);
        });
        yield Airship.show(bridge => (React.createElement(TextInputModal, { bridge: bridge, title: sprintf(lstrings.recovery_answer, index + 1), inputLabel: lstrings.recovery_answer_placeholder, returnKeyType: "go", autoCapitalize: "none", autoCorrect: false, autoFocus: true, onSubmit: validateAnswer }))).then(text => {
            if (text == null)
                return;
            answers[index] = text;
            setAnswers([...answers]);
        });
    });
    const renderSaveRecoveryMessage = (showEmailPrompt) => {
        return (React.createElement(ModalMessage, null,
            lstrings.recovery_save_hint_token + '\n\n',
            React.createElement(Text, { style: styles.warningText }, lstrings.recovery_save_hint_username_answers + '\n\n'),
            showEmailPrompt ? lstrings.recovery_save_email_prompt : null));
    };
    const saveRecovery = (emailAddress) => __awaiter(void 0, void 0, void 0, function* () {
        const { username } = account;
        if (username == null)
            return;
        const okQuestions = questions.filter(stringPredicate);
        const okAnswers = answers.filter(stringPredicate);
        try {
            const recoveryKey = yield account.changeRecovery(okQuestions, okAnswers);
            if (emailAddress == null) {
                yield shareRecovery(username, recoveryKey, branding);
            }
            else {
                yield sendRecoveryEmail(emailAddress, username, recoveryKey, branding);
            }
            yield Airship.show(bridge => (React.createElement(ButtonsModal, { bridge: bridge, title: lstrings.recovery_success_title, message: lstrings.recovery_success_message, buttons: { ok: { label: lstrings.ok } } })));
            setAnswers([]);
            onComplete();
        }
        catch (error) {
            showError(error);
        }
    });
    const saveRecoveryViaEmail = () => __awaiter(void 0, void 0, void 0, function* () {
        const emailAddress = yield Airship.show(bridge => (React.createElement(TextInputModal, { bridge: bridge, title: lstrings.save_recovery_token, inputLabel: lstrings.recovery_complete_enter_email_label, message: renderSaveRecoveryMessage(true), submitLabel: lstrings.next_label, keyboardType: "email-address", autoFocus: false, autoCapitalize: "none", returnKeyType: "go", onSubmit: (email) => __awaiter(void 0, void 0, void 0, function* () {
                return validateEmail(email) ? true : lstrings.recovery_invalid_email;
            }) })));
        if (emailAddress == null)
            return;
        yield saveRecovery(emailAddress);
    });
    const saveRecoveryViaShare = () => __awaiter(void 0, void 0, void 0, function* () {
        yield Airship.show(bridge => (React.createElement(ButtonsModal, { bridge: bridge, title: lstrings.save_recovery_token, buttons: {
                ok: { label: lstrings.ok },
                cancel: { label: lstrings.cancel, type: 'secondary' }
            } }, renderSaveRecoveryMessage(false)))).then((button) => __awaiter(void 0, void 0, void 0, function* () {
            if (button !== 'ok')
                return;
            yield saveRecovery();
        }));
    });
    const changeRecovery = () => {
        Keyboard.dismiss();
        try {
            Airship.show(bridge => (React.createElement(ButtonsModal, { bridge: bridge, title: lstrings.recovery_change_title, message: lstrings.recovery_change_message, buttons: {
                    ok: {
                        label: lstrings.ok
                    },
                    cancel: {
                        label: lstrings.cancel,
                        type: 'secondary'
                    }
                } }))).then((button) => __awaiter(void 0, void 0, void 0, function* () {
                if (button !== 'ok')
                    return;
                setRecoveryLocked(false);
                setAnswers([]);
            }));
        }
        catch (error) {
            showError(error);
        }
    };
    const deleteRecovery = () => {
        Keyboard.dismiss();
        try {
            Airship.show(bridge => (React.createElement(ButtonsModal, { bridge: bridge, title: lstrings.recovery_reset_confirm_title, message: lstrings.recovery_reset_confirm_message, buttons: {
                    confirm: { label: lstrings.confirm },
                    cancel: { label: lstrings.cancel, type: 'secondary' }
                } }))).then((button) => __awaiter(void 0, void 0, void 0, function* () {
                if (button !== 'confirm')
                    return;
                yield account.deleteRecovery();
                onComplete();
            }));
        }
        catch (error) {
            showError(error);
        }
    };
    const renderQuestionAnswer = (index) => {
        var _a, _b;
        const question = (_a = extractQuestion(questions[index])) !== null && _a !== void 0 ? _a : questionPrompt;
        const answer = (_b = answers[index]) !== null && _b !== void 0 ? _b : answerPrompt;
        return (React.createElement(View, null,
            React.createElement(Tile, { type: "touchable", title: sprintf(lstrings.recovery_question, index + 1), body: question !== null && question !== void 0 ? question : questionPrompt, onPress: handleQuestion(index), contentPadding: false, disabled: recoveryLocked }),
            React.createElement(Tile, { type: questions[index] == null ? 'static' : 'touchable', title: sprintf(lstrings.recovery_answer, index + 1), body: answer, onPress: handleAnswer(index), contentPadding: false, disabled: recoveryLocked || questions[index] == null })));
    };
    const renderCaseSensitivityWarning = () => {
        return showCaseSensitivityWarning && !recoveryLocked ? (React.createElement(WarningCard, { title: lstrings.answer_case_sensitive, marginRem: 1 })) : null;
    };
    // TODO: Need to rework buttons for UI4
    const renderModifyButtons = () => {
        return (React.createElement(View, { style: styles.buttonsContainer },
            React.createElement(MainButton, { marginRem: 0, label: lstrings.recovery_change_button, onPress: changeRecovery, type: "primary" }),
            React.createElement(EdgeTouchableOpacity, { onPress: deleteRecovery },
                React.createElement(View, { style: styles.disableButtonContainer },
                    React.createElement(Text, { numberOfLines: 1, style: styles.disableButton }, lstrings.recovery_disable_button)))));
    };
    const renderConfirmButtons = () => {
        return (React.createElement(View, { style: styles.buttonsContainer },
            React.createElement(MainButton, { label: lstrings.confirm_email, marginRem: 0.25, onPress: saveRecoveryViaEmail, disabled: !confirmButtonsEnabled, type: "primary" }),
            React.createElement(MainButton, { label: lstrings.confirm_share, marginRem: 0.25, onPress: saveRecoveryViaShare, disabled: !confirmButtonsEnabled, type: "primary" }),
            React.createElement(MainButton, { label: lstrings.cancel, marginRem: [0.25, 0, 1], onPress: onComplete, type: "escape" })));
    };
    const renderButtons = () => {
        return recoveryLocked ? renderModifyButtons() : renderConfirmButtons();
    };
    const renderQuestionAnswers = () => {
        return (React.createElement(View, null,
            renderQuestionAnswer(0),
            renderQuestionAnswer(1)));
    };
    return (React.createElement(ThemedScene, { paddingRem: 0 },
        React.createElement(ScrollView, { ref: scrollViewRef, contentContainerStyle: styles.content },
            renderQuestionAnswers(),
            renderCaseSensitivityWarning(),
            renderButtons())));
};
const getStyles = cacheStyles((theme) => ({
    content: {
        flex: 1,
        justifyContent: 'space-between',
        flexDirection: 'column'
    },
    buttonsContainer: {
        marginHorizontal: theme.rem(1)
    },
    disableButtonContainer: {
        alignSelf: 'center',
        marginVertical: theme.rem(1)
    },
    disableButton: {
        color: theme.dangerText,
        marginBottom: theme.rem(1),
        fontSize: theme.rem(1),
        fontFamily: theme.fontFaceDefault
    },
    warningText: {
        color: theme.warningText
    }
}));
