var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as React from 'react';
import { ActivityIndicator, ScrollView, View } from 'react-native';
import { cacheStyles } from 'react-native-patina';
import AntDesignIcon from 'react-native-vector-icons/AntDesign';
import FontAwesome from 'react-native-vector-icons/FontAwesome';
import { submitLogin } from '../../../actions/LoginCompleteActions';
import { lstrings } from '../../../common/locales/strings';
import { useImports } from '../../../hooks/useImports';
import { useDispatch } from '../../../types/ReduxTypes';
import { toLocalTime } from '../../../util/utils';
import { showError } from '../../services/AirshipInstance';
import { useTheme } from '../../services/ThemeContext';
import { IconHeaderRow } from '../../themed/IconHeaderRow';
import { LinkRow } from '../../themed/LinkRow';
import { MainButton } from '../../themed/MainButton';
import { ThemedScene } from '../../themed/ThemedScene';
import { MessageText, Warning } from '../../themed/ThemedText';
export class SecurityAlertsSceneComponent extends React.Component {
    constructor(props) {
        super(props);
        this.handleApproveReset = () => {
            const { account } = this.props;
            this.setState({ spinReset: true });
            account
                .disableOtp()
                .catch(error => showError(error))
                .then(() => this.setState({
                showSkip: true,
                spinReset: false
            }));
        };
        this.handleApproveVoucher = (voucherId) => {
            const { account } = this.props;
            this.setState(state => ({
                spinVoucher: Object.assign(Object.assign({}, state.spinVoucher), { [voucherId]: true })
            }));
            const { approveVoucher = nopVoucher } = account;
            approveVoucher(voucherId)
                .catch(error => showError(error))
                .then(() => this.setState(state => ({
                showSkip: true,
                spinVoucher: Object.assign(Object.assign({}, state.spinVoucher), { [voucherId]: false })
            })));
        };
        this.handleDeny = () => __awaiter(this, void 0, void 0, function* () {
            const { account } = this.props;
            const { otpResetDate, pendingVouchers } = this.state;
            this.setState({ needsResecure: true });
            const { rejectVoucher = nopVoucher } = account;
            const promises = pendingVouchers.map((voucher) => __awaiter(this, void 0, void 0, function* () { return yield rejectVoucher(voucher.voucherId); }));
            if (otpResetDate != null) {
                promises.push(account.cancelOtpReset());
            }
            try {
                yield Promise.all(promises);
            }
            finally {
                this.setState({ showSkip: true });
            }
        });
        this.handleSkip = () => {
            const { onDone } = this.props;
            onDone();
        };
        this.checkEmpty = () => {
            const { account, startResecure, onDone } = this.props;
            const { needsResecure, otpResetDate, pendingVouchers } = this.state;
            if (otpResetDate == null && pendingVouchers.length <= 0) {
                if (needsResecure)
                    startResecure(account);
                else
                    onDone();
            }
        };
        const { otpResetDate, pendingVouchers = [] } = props.account;
        this.state = {
            needsResecure: props.account.recoveryLogin,
            otpResetDate: otpResetDate != null ? new Date(otpResetDate) : undefined,
            pendingVouchers,
            showSkip: false,
            spinReset: false,
            spinVoucher: {}
        };
    }
    componentDidMount() {
        const { account } = this.props;
        this.checkEmpty();
        this.cleanups = [
            account.watch('otpResetDate', otpResetDate => {
                const date = otpResetDate != null ? new Date(otpResetDate) : undefined;
                this.setState({ otpResetDate: date }, this.checkEmpty);
            }),
            account.watch('pendingVouchers', pendingVouchers => this.setState({ pendingVouchers }, this.checkEmpty))
        ];
    }
    componentWillUnmount() {
        if (this.cleanups != null)
            this.cleanups.forEach(f => f());
    }
    render() {
        const { otpResetDate, pendingVouchers, showSkip } = this.state;
        const count = pendingVouchers.length + (otpResetDate != null ? 1 : 0);
        return (React.createElement(ThemedScene, null,
            React.createElement(ScrollView, { style: { flex: 1 } },
                React.createElement(IconHeaderRow, { renderIcon: theme => (React.createElement(FontAwesome, { name: "exclamation-triangle", size: theme.rem(2.5) })) },
                    React.createElement(MessageText, null, count > 1
                        ? lstrings.alert_scene_message_many
                        : lstrings.alert_scene_message)),
                React.createElement(MessageText, null,
                    React.createElement(Warning, null, lstrings.alert_scene_warning)),
                this.renderVouchers(),
                this.renderReset(),
                showSkip ? (React.createElement(LinkRow, { label: lstrings.skip_button, onPress: this.handleSkip, renderIcon: () => null })) : null),
            React.createElement(MainButton, { label: lstrings.alert_scene_deny, marginRem: 1, onPress: this.handleDeny })));
    }
    renderReset() {
        const { theme } = this.props;
        const { otpResetDate, spinReset } = this.state;
        const styles = getStyles(theme);
        if (otpResetDate == null)
            return null;
        return (React.createElement(View, { style: styles.tile },
            React.createElement(MessageText, null, lstrings.alert_scene_reset_message),
            React.createElement(MessageText, null,
                lstrings.alert_scene_reset_date,
                otpResetDate.toLocaleString()),
            spinReset ? (React.createElement(ActivityIndicator, { color: theme.primaryButton[0], style: styles.cardSpinner })) : (React.createElement(LinkRow, { label: lstrings.alert_scene_approve, onPress: this.handleApproveReset, renderIcon: theme => (React.createElement(AntDesignIcon, { name: "check", size: theme.rem(1) })) }))));
    }
    renderVouchers() {
        const { theme } = this.props;
        const { pendingVouchers, spinVoucher } = this.state;
        const styles = getStyles(theme);
        return pendingVouchers.map(voucher => (React.createElement(View, { style: styles.tile, key: voucher.voucherId },
            React.createElement(MessageText, null,
                voucher.deviceDescription != null
                    ? lstrings.alert_scene_device + voucher.deviceDescription + '\n'
                    : null,
                lstrings.alert_scene_ip + voucher.ipDescription + '\n',
                lstrings.alert_scene_date + toLocalTime(voucher.created)),
            React.createElement(MessageText, null, lstrings.alert_scene_reset_date + toLocalTime(voucher.activates)),
            spinVoucher[voucher.voucherId] ? (React.createElement(ActivityIndicator, { color: theme.primaryButton[0], style: styles.cardSpinner })) : (React.createElement(LinkRow, { label: lstrings.alert_scene_approve, onPress: () => this.handleApproveVoucher(voucher.voucherId), renderIcon: theme => (React.createElement(AntDesignIcon, { name: "check", size: theme.rem(1) })) })))));
    }
}
function nopVoucher(voucherId) {
    return __awaiter(this, void 0, void 0, function* () {
        return yield Promise.resolve();
    });
}
const getStyles = cacheStyles((theme) => ({
    tile: {
        backgroundColor: theme.tileBackground,
        borderRadius: theme.rem(0.5),
        margin: theme.rem(0.5),
        padding: theme.rem(0.5)
    },
    cardSpinner: {
        alignSelf: 'flex-end',
        height: theme.rem(1.5),
        margin: theme.rem(0.5)
    }
}));
export function SecurityAlertsScene(props) {
    const { route } = props;
    const { account } = route.params;
    const { onComplete = () => { }, onLogin } = useImports();
    const dispatch = useDispatch();
    const theme = useTheme();
    const handleDone = () => {
        if (onLogin != null)
            dispatch(submitLogin(account));
        else
            onComplete();
    };
    const handleResecure = () => {
        dispatch({
            type: 'NAVIGATE',
            data: { name: 'resecurePassword', params: { account } }
        });
    };
    return (React.createElement(SecurityAlertsSceneComponent, { account: account, route: route, startResecure: handleResecure, theme: theme, onDone: handleDone }));
}
