/**
 * IMPORTANT: Changes in this file MUST be synced between edge-react-gui and
 * edge-login-ui-rn!
 */
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import * as React from 'react';
import { useMemo } from 'react';
import { ActivityIndicator, Platform, Text, TextInput, TouchableOpacity, View } from 'react-native';
import Animated, { interpolate, interpolateColor, useAnimatedRef, useAnimatedStyle, useDerivedValue, useSharedValue, withDelay, withTiming } from 'react-native-reanimated';
import { useHandler } from '../../hooks/useHandler';
import { useSpaceStyle } from '../../hooks/useSpaceStyle';
import { EdgeTouchableWithoutFeedback } from '../common/EdgeTouchableWithoutFeedback';
import { styled, styledWithRef } from '../hoc/styled';
import { CloseIconAnimated, EyeIconAnimated } from '../icons/ThemedIcons';
import { useTheme } from '../services/ThemeContext';
import { NumericInput } from './NumericInput';
const AnimatedTextInput = Animated.createAnimatedComponent(TextInput);
const isAndroid = Platform.OS === 'android';
export const FilledTextInput = React.forwardRef((props, ref) => {
    var _a;
    const { 
    // Contents:
    error, placeholder, valid, value, numeric, minDecimals, maxDecimals, 
    // Appearance:
    clearIcon = true, iconComponent, multiline = false, scale: scaleProp, showSpinner = false, prefix, suffix, 
    // Callbacks:
    onBlur, onChangeText, onClear, onFocus, onSubmitEditing, 
    // TextInput:
    autoCapitalize = props.secureTextEntry === true ? 'none' : undefined, autoCorrect, autoFocus = false, blurOnClear = false, blurOnSubmit, disabled = false, inputAccessoryViewID, keyboardType, maxLength, returnKeyType, secureTextEntry, testID, textsizeRem } = props, spaceProps = __rest(props, ["error", "placeholder", "valid", "value", "numeric", "minDecimals", "maxDecimals", "clearIcon", "iconComponent", "multiline", "scale", "showSpinner", "prefix", "suffix", "onBlur", "onChangeText", "onClear", "onFocus", "onSubmitEditing", "autoCapitalize", "autoCorrect", "autoFocus", "blurOnClear", "blurOnSubmit", "disabled", "inputAccessoryViewID", "keyboardType", "maxLength", "returnKeyType", "secureTextEntry", "testID", "textsizeRem"]);
    const theme = useTheme();
    const themeRem = theme.rem(1);
    const LeftIcon = iconComponent;
    const hasIcon = LeftIcon != null;
    const hasValue = value !== '';
    const spaceStyle = useSpaceStyle(spaceProps);
    // Show/Hide password input:
    const [hidePassword, setHidePassword] = React.useState(secureTextEntry !== null && secureTextEntry !== void 0 ? secureTextEntry : false);
    const handleHidePassword = () => setHidePassword(!hidePassword);
    // Imperative methods:
    const inputRef = useAnimatedRef();
    function blur() {
        if (inputRef.current != null)
            inputRef.current.blur();
    }
    function clear() {
        if (inputRef.current != null)
            inputRef.current.clear();
        if (onChangeText != null)
            onChangeText('');
        if (blurOnClear || !hasValue)
            blur();
        if (onClear != null)
            onClear();
    }
    function focus() {
        if (inputRef.current != null && !disabled)
            inputRef.current.focus();
    }
    function isFocused() {
        return inputRef.current != null ? inputRef.current.isFocused() : false;
    }
    function setNativeProps(nativeProps) {
        if (inputRef.current != null)
            inputRef.current.setNativeProps(nativeProps);
    }
    React.useImperativeHandle(ref, () => ({
        blur,
        clear,
        focus,
        isFocused,
        setNativeProps
    }));
    // Animates between 0 and 1 based our disabled state:
    const disableAnimation = useSharedValue(0);
    React.useEffect(() => {
        disableAnimation.value = withTiming(disabled ? 1 : 0);
    }, [disableAnimation, disabled]);
    // Animates between 0 and 1 based on focus:
    const baseDuration = 300;
    const focusAnimation = useSharedValue(0);
    // A delayed focus animation is required for closing the top border when
    // animating everything back to their original positions.
    const animationDelay = 0.4 * baseDuration;
    const handleBlur = useHandler(() => {
        focusAnimation.value = withDelay(animationDelay, withTiming(0, { duration: baseDuration }));
        if (onBlur != null)
            onBlur();
    });
    const handleChangeText = useHandler((value) => {
        if (props.onChangeText != null)
            props.onChangeText(value);
    });
    const handleClearPress = useHandler(() => {
        clear();
    });
    const handleFocus = useHandler(() => {
        focusAnimation.value = withTiming(1, { duration: baseDuration });
        if (onFocus != null)
            onFocus();
    });
    const handleSubmitEditing = useHandler(() => {
        if (onSubmitEditing != null)
            onSubmitEditing();
    });
    const leftIconSize = useDerivedValue(() => hasIcon
        ? hasValue
            ? 0
            : interpolate(focusAnimation.value, [0, 1], [themeRem, 0])
        : 0, [hasIcon, hasValue]);
    const rightIconSize = useDerivedValue(() => clearIcon ? (hasValue ? themeRem : focusAnimation.value * themeRem) : 0, [clearIcon, hasValue]);
    const scale = useDerivedValue(() => { var _a; return (_a = scaleProp === null || scaleProp === void 0 ? void 0 : scaleProp.value) !== null && _a !== void 0 ? _a : 1; });
    const interpolateIconColor = useAnimatedColorInterpolateFn(theme.textInputIconColor, theme.textInputIconColorFocused, theme.textInputIconColorDisabled);
    const iconColor = useDerivedValue(() => interpolateIconColor(focusAnimation, disableAnimation));
    const focusValue = useDerivedValue(() => (hasValue ? 1 : focusAnimation.value), [hasValue]);
    // Character Limit:
    const charactersLeft = maxLength === undefined ? '' : `${maxLength - value.length}`;
    const InputComponent = numeric ? StyledNumericInput : StyledAnimatedTextInput;
    // HACK: Some Android devices/versions, mostly Samsung, have a bug where the
    // text input always blurs immediately after focusing.
    const hackKeyboardType = isAndroid &&
        !secureTextEntry && // Don't apply this hack if we are intentionally doing a secure text entry
        !hidePassword &&
        (keyboardType == null || keyboardType === 'default')
        ? 'visible-password'
        : keyboardType;
    return (React.createElement(View, { style: spaceStyle },
        React.createElement(EdgeTouchableWithoutFeedback, { accessible: false, testID: testID, onPress: () => focus() },
            React.createElement(Container, { disableAnimation: disableAnimation, focusAnimation: focusAnimation, multiline: multiline, scale: scale },
                React.createElement(SideContainer, { scale: leftIconSize }, LeftIcon == null ? null : (React.createElement(LeftIcon, { color: iconColor, size: leftIconSize }))),
                React.createElement(InnerContainer, { focusValue: focusValue, hasPlaceholder: placeholder != null },
                    placeholder == null ? null : (React.createElement(Placeholder, { shift: focusValue },
                        React.createElement(PlaceholderText, { disableAnimation: disableAnimation, focusAnimation: focusAnimation, scale: scale, shift: focusValue, textsizeRem: textsizeRem }, placeholder))),
                    prefix == null ? null : (React.createElement(PrefixAnimatedText, { visibility: focusValue }, prefix)),
                    React.createElement(InputComponent, { accessible: true, animated: true, editable: !disabled, ref: inputRef, keyboardType: hackKeyboardType, returnKeyType: returnKeyType, accessibilityState: { disabled }, autoFocus: autoFocus, disableAnimation: disableAnimation, focusAnimation: focusAnimation, minDecimals: minDecimals, maxDecimals: maxDecimals, multiline: multiline, selectionColor: theme.textInputTextColor, testID: `${testID}.textInput`, textAlignVertical: "top", textsizeRem: textsizeRem, scale: scale, value: value, 
                        // Callbacks:
                        onBlur: handleBlur, onChangeText: handleChangeText, onFocus: handleFocus, onSubmitEditing: handleSubmitEditing, maxLength: maxLength, 
                        // Other Props:
                        autoCapitalize: autoCapitalize, autoCorrect: autoCorrect, blurOnSubmit: blurOnSubmit, inputAccessoryViewID: inputAccessoryViewID, secureTextEntry: hidePassword }),
                    suffix == null ? null : React.createElement(SuffixText, null, suffix)),
                showSpinner ? React.createElement(ActivityIndicator, null) : null,
                secureTextEntry ? (React.createElement(TouchContainer, { extendTappable: "leftOnly", testID: `${testID}.eyeIcon`, onPress: handleHidePassword },
                    React.createElement(IconContainer, null,
                        React.createElement(EyeIconAnimated, { accessible: true, color: iconColor, off: !hidePassword })))) : null,
                React.createElement(TouchContainer, { extendTappable: secureTextEntry ? 'rightOnly' : 'full', accessible: true, onPress: handleClearPress, testID: `${testID}.clearIcon` },
                    React.createElement(SideContainer, { scale: rightIconSize },
                        React.createElement(CloseIconAnimated, { color: iconColor, size: rightIconSize }))))),
        valid != null || error != null || charactersLeft !== '' ? (React.createElement(MessagesContainer, { noLayoutFlow: charactersLeft === '' },
            React.createElement(Message, { danger: error != null }, (_a = valid !== null && valid !== void 0 ? valid : error) !== null && _a !== void 0 ? _a : null),
            React.createElement(Message, null, charactersLeft))) : null));
});
const Container = styled(Animated.View)(theme => ({ disableAnimation, focusAnimation, multiline, scale }) => {
    const rem = theme.rem(1);
    const interpolateInputBackgroundColor = useAnimatedColorInterpolateFn(theme.textInputBackgroundColor, theme.textInputBackgroundColorFocused, theme.textInputBackgroundColorDisabled);
    const interpolateOutlineColor = useAnimatedColorInterpolateFn(theme.textInputBorderColor, theme.textInputBorderColorFocused, theme.textInputBorderColorDisabled);
    return [
        {
            flex: multiline ? 1 : undefined,
            alignItems: multiline ? 'stretch' : 'center',
            borderWidth: theme.textInputBorderWidth,
            borderRadius: theme.rem(0.5),
            flexDirection: 'row',
            paddingHorizontal: theme.rem(0.75)
        },
        useAnimatedStyle(() => ({
            backgroundColor: interpolateInputBackgroundColor(focusAnimation, disableAnimation),
            borderColor: interpolateOutlineColor(focusAnimation, disableAnimation),
            opacity: interpolate(scale.value, [1, 0.5], [1, 0]),
            marginHorizontal: interpolate(scale.value, [1, 0], [0, 2 * rem]),
            paddingVertical: scale.value * rem
        }))
    ];
});
/**
 * extendTappable: Which horizontal side of the icon do we want to increase
 * tappable area? 'full' means both left and right sides.
 */
const TouchContainer = styled(TouchableOpacity)(theme => ({ extendTappable }) => {
    // Increase tappable area with padding, while net 0 with negative margin to
    // visually appear as if 0 margins/padding
    const tapArea = extendTappable === 'leftOnly'
        ? {
            paddingLeft: theme.rem(1),
            marginLeft: -theme.rem(1)
        }
        : extendTappable === 'rightOnly'
            ? {
                paddingRight: theme.rem(1),
                marginRight: -theme.rem(1)
            }
            : // extendTappable === 'full'
                {
                    marginHorizontal: -theme.rem(1),
                    paddingHorizontal: theme.rem(1)
                };
    return Object.assign({ paddingVertical: theme.rem(1.25), marginVertical: -theme.rem(1.25) }, tapArea);
});
const IconContainer = styled(View)(theme => ({
    paddingHorizontal: theme.rem(0.25)
}));
const SideContainer = styled(Animated.View)(theme => ({ scale }) => {
    return [
        {
            alignItems: 'stretch',
            aspectRatio: 1
        },
        useAnimatedStyle(() => ({
            width: scale.value,
            opacity: scale.value
        }))
    ];
});
const InnerContainer = styled(Animated.View)(theme => ({ hasPlaceholder, focusValue }) => {
    const rem = theme.rem(1);
    // Need 2 pixels of shift given a 16 point rem settings
    const androidHShift = isAndroid ? rem / 8 : 0;
    return [
        {
            left: androidHShift,
            flex: 1,
            flexDirection: 'row',
            alignItems: 'flex-start',
            alignSelf: 'flex-start'
        },
        useAnimatedStyle(() => {
            const shiftValue = interpolate(focusValue.value, [0, 1], [0, rem * 0.5]);
            return {
                marginTop: hasPlaceholder ? shiftValue : undefined,
                marginBottom: hasPlaceholder ? -shiftValue : undefined
            };
        })
    ];
});
const PrefixAnimatedText = styled(Animated.Text)(theme => ({ visibility }) => {
    const rem = theme.rem(1);
    return [
        {
            color: theme.secondaryText,
            fontFamily: theme.fontFaceDefault,
            fontSize: theme.rem(1),
            includeFontPadding: false
        },
        useAnimatedStyle(() => {
            return {
                opacity: visibility.value,
                transform: [
                    { translateY: (1 - visibility.value) * rem },
                    { scale: visibility.value }
                ]
            };
        })
    ];
});
const SuffixText = styled(Text)(theme => {
    return {
        color: theme.secondaryText,
        fontFamily: theme.fontFaceDefault,
        fontSize: theme.rem(1),
        includeFontPadding: false,
        marginRight: theme.rem(0.5)
    };
});
const Placeholder = styled(Animated.View)(theme => ({ shift }) => {
    const rem = theme.rem(1);
    const androidVShift = isAndroid ? rem / 16 : 0;
    return [
        {
            position: 'absolute',
            top: androidVShift,
            left: rem * 0.4,
            alignItems: 'center',
            justifyContent: 'center',
            paddingHorizontal: theme.rem(0),
            paddingVertical: 0,
            margin: 0
        },
        useAnimatedStyle(() => ({
            transform: [
                {
                    translateY: interpolate(shift.value, [0, 1], [0, -1 * rem])
                },
                {
                    translateX: interpolate(shift.value, [0, 1], [0, -0.4 * rem])
                }
            ]
        }))
    ];
});
const PlaceholderText = styled(Animated.Text)(theme => ({ disableAnimation, focusAnimation, scale, shift, textsizeRem }) => {
    const fontSizeBase = theme.rem(textsizeRem !== null && textsizeRem !== void 0 ? textsizeRem : scale.value);
    const fontSizeScaled = theme.rem(scale.value) * 0.75;
    const interpolatePlaceholderTextColor = useAnimatedColorInterpolateFn(theme.textInputPlaceholderColor, theme.textInputPlaceholderColorFocused, theme.textInputPlaceholderColorDisabled);
    return [
        {
            fontFamily: theme.fontFaceDefault,
            fontSize: theme.rem(1),
            includeFontPadding: false
        },
        useAnimatedStyle(() => {
            return {
                color: interpolatePlaceholderTextColor(focusAnimation, disableAnimation),
                fontSize: interpolate(shift.value, [0, 1], [fontSizeBase, fontSizeScaled])
            };
        })
    ];
});
const StyledAnimatedTextInput = styledWithRef(AnimatedTextInput)(theme => ({ disableAnimation, focusAnimation, scale, textsizeRem }) => {
    const rem = theme.rem(textsizeRem !== null && textsizeRem !== void 0 ? textsizeRem : 1);
    const interpolateTextColor = useAnimatedColorInterpolateFn(theme.textInputTextColor, theme.textInputTextColorFocused, theme.textInputTextColorDisabled);
    // Need 2 pixels of shift given a 16 point rem settings
    // This is due to Android rendering a text input vertically lower
    // than a Text field by ~2 pixels
    const androidVShift = isAndroid ? rem / 8 : 0;
    return [
        {
            color: theme.textInputBackgroundColor,
            flexGrow: 1,
            flexShrink: 1,
            fontFamily: theme.fontFaceDefault,
            paddingHorizontal: 0,
            paddingVertical: 0,
            transform: [{ translateY: -androidVShift }],
            margin: 0
        },
        useAnimatedStyle(() => ({
            color: interpolateTextColor(focusAnimation, disableAnimation),
            fontSize: scale.value * rem
        }))
    ];
});
const StyledNumericInput = styledWithRef(NumericInput)(theme => ({ disableAnimation, focusAnimation, textsizeRem, scale }) => {
    const rem = theme.rem(textsizeRem !== null && textsizeRem !== void 0 ? textsizeRem : 1);
    const interpolateTextColor = useAnimatedColorInterpolateFn(theme.textInputTextColor, theme.textInputTextColorFocused, theme.textInputTextColorDisabled);
    // Need 2 pixels of shift given a 16 point rem settings
    // This is due to Android rendering a text input vertically lower
    // than a Text field by ~2 pixels
    const androidVShift = isAndroid ? rem / 8 : 0;
    return [
        {
            color: theme.textInputBackgroundColor,
            flexGrow: 1,
            flexShrink: 1,
            fontFamily: theme.fontFaceDefault,
            paddingHorizontal: 0,
            paddingVertical: 0,
            transform: [{ translateY: -androidVShift }],
            margin: 0
        },
        useAnimatedStyle(() => ({
            color: interpolateTextColor(focusAnimation, disableAnimation),
            fontSize: scale.value * rem
        }))
    ];
});
const MessagesContainer = styled(Animated.View)(theme => props => [
    {
        flexDirection: 'row',
        justifyContent: 'space-between',
        paddingHorizontal: theme.rem(0.5),
        height: theme.rem(1)
    },
    props.noLayoutFlow
        ? {
            // HACK: If this field has a potential error message, counter-act the
            // layout flow to avoid the effect of the error message's appearance
            // pushing components below the this text field down.
            // If there's a counter, this field is already taking up the maximum
            // amount of vertical space, so the above is not an issue.
            marginBottom: -theme.rem(1)
        }
        : {}
]);
const Message = styled(Text)(theme => props => [
    {
        color: props.danger === true ? theme.dangerText : theme.secondaryText,
        fontFamily: theme.fontFaceDefault,
        fontSize: theme.rem(0.75),
        includeFontPadding: false
    }
]);
function useAnimatedColorInterpolateFn(fromColor, toColor, disabledColor) {
    const interpolateFn = useMemo(() => {
        return (focusValue, disabledValue) => {
            'worklet';
            const interFocusColor = interpolateColor(focusValue.value, [0, 1], [fromColor, toColor]);
            return interpolateColor(disabledValue.value, [0, 1], [interFocusColor, disabledColor]);
        };
    }, [fromColor, toColor, disabledColor]);
    return interpolateFn;
}
