/**
 * IMPORTANT: Changes in this file MUST be synced with edge-react-gui!
 */
import * as React from 'react';
import { Platform, ScrollView, Text, View } from 'react-native';
import { cacheStyles } from 'react-native-patina';
import AntDesignIcon from 'react-native-vector-icons/AntDesign';
import { lstrings } from '../../common/locales/strings';
import { fixSides, mapSides, sidesToPadding } from '../../util/sides';
import { EdgeTouchableOpacity } from '../common/EdgeTouchableOpacity';
import { GradientFadeOut } from '../modals/GradientFadeout';
import { useTheme } from '../services/ThemeContext';
export function ModalTitle(props) {
    const { center, children, icon = null, paddingRem } = props;
    const theme = useTheme();
    const styles = getStyles(theme);
    const padding = sidesToPadding(mapSides(fixSides(paddingRem, 0), theme.rem));
    const androidAdjust = Platform.OS === 'android' ? styles.androidAdjust : null;
    const centerStyle = center ? styles.titleCenter : null;
    return (React.createElement(View, { style: styles.titleContainer },
        icon ? React.createElement(View, { style: styles.titleIconContainer }, icon) : null,
        React.createElement(Text, { style: [styles.titleText, centerStyle, padding, androidAdjust] }, children)));
}
export function ModalMessage(props) {
    const { children, isWarning, paddingRem } = props;
    const theme = useTheme();
    const styles = getStyles(theme);
    const padding = sidesToPadding(mapSides(fixSides(paddingRem, 0), theme.rem));
    const warningStyle = isWarning ? styles.warningText : null;
    const androidAdjust = Platform.OS === 'android' ? styles.androidAdjust : null;
    return (React.createElement(Text, { style: [styles.messageText, padding, warningStyle, androidAdjust] }, children));
}
/**
 * Renders a close button
 */
export function ModalFooter(props) {
    const theme = useTheme();
    const styles = getStyles(theme);
    return (React.createElement(EdgeTouchableOpacity, { onPress: props.onPress, style: styles.closeContainer },
        React.createElement(AntDesignIcon, { accessibilityHint: lstrings.modal_close_hint, color: theme.iconTappable, name: "close", size: theme.rem(1.25) })));
}
ModalFooter.bottomRem = 3;
/**
 * A consistently styled scroll area for use in modals. Should only be used
 * within ThemedModal.
 */
export function ModalScrollArea(props) {
    const { children } = props;
    const theme = useTheme();
    const styles = getStyles(theme);
    return (React.createElement(View, { style: styles.scrollContainer },
        React.createElement(ScrollView, { contentContainerStyle: styles.scrollPadding }, children),
        React.createElement(ModalFooterFade, null)));
}
/**
 * For fading the bottom of the modal if the modal caller has its own special
 * scroll implementation and does not use the ThemedModal's built-in 'scroll'
 * prop
 */
export const ModalFooterFade = () => {
    const theme = useTheme();
    const styles = getStyles(theme);
    return (React.createElement(View, { style: styles.footerFadeContainer },
        React.createElement(GradientFadeOut, null)));
};
const getStyles = cacheStyles((theme) => ({
    androidAdjust: {
        top: -1
    },
    closeContainer: {
        alignItems: 'center',
        padding: theme.rem(1),
        marginBottom: theme.rem(-1)
    },
    scrollContainer: {
        marginBottom: theme.rem(-ModalFooter.bottomRem + 0.5)
    },
    scrollPadding: {
        paddingBottom: theme.rem(ModalFooter.bottomRem)
    },
    titleContainer: {
        alignItems: 'center',
        flexDirection: 'row',
        marginHorizontal: theme.rem(0.5)
    },
    titleIconContainer: {
        marginRight: theme.rem(0.5)
    },
    titleText: {
        color: theme.primaryText,
        fontFamily: theme.fontFaceMedium,
        fontSize: theme.rem(1.2)
    },
    titleCenter: {
        textAlign: 'center',
        flexGrow: 1
    },
    warningText: {
        color: theme.warningText
    },
    messageText: {
        color: theme.primaryText,
        fontFamily: theme.fontFaceDefault,
        fontSize: theme.rem(1),
        margin: theme.rem(0.5),
        textAlign: 'left'
    },
    footerFadeContainer: {
        position: 'absolute',
        bottom: 0,
        left: 0,
        right: 0,
        zIndex: 1
    }
}));
