import * as React from 'react';
import { View } from 'react-native';
import { cacheStyles } from 'react-native-patina';
import FontAwesome from 'react-native-vector-icons/FontAwesome';
import SimpleLineIcons from 'react-native-vector-icons/SimpleLineIcons';
import { lstrings } from '../../common/locales/strings';
import { EdgeAnim } from '../common/EdgeAnim';
import { useTheme } from '../services/ThemeContext';
import { EdgeText } from './EdgeText';
import { IconSignal } from './IconSignal';
export const PasswordStatus = (props) => {
    const { passwordReqs } = props;
    const theme = useTheme();
    const styles = getStyles(theme);
    const { minLengthMet, hasNumber, hasLowercase, hasUppercase, confirmationMatches } = passwordReqs;
    const list = [
        { title: lstrings.must_ten_characters, validationStatus: minLengthMet },
        {
            title: lstrings.must_one_lowercase,
            validationStatus: hasLowercase
        },
        {
            title: lstrings.must_one_uppercase,
            validationStatus: hasUppercase
        },
        { title: lstrings.must_one_number, validationStatus: hasNumber },
        {
            title: lstrings.password_must_match,
            validationStatus: confirmationMatches
        }
    ];
    const passwordValid = !list.some(({ validationStatus }) => validationStatus === 'unmet' || validationStatus === 'error');
    return (React.createElement(EdgeAnim, { enter: { type: 'fadeInUp', distance: 100 }, style: [styles.container, passwordValid && styles.passedContainer] },
        React.createElement(View, { style: styles.top },
            React.createElement(IconSignal, { enabled: passwordValid, enabledIcon: props => React.createElement(FontAwesome, Object.assign({}, props, { name: "check-circle" })), disabledIcon: props => React.createElement(SimpleLineIcons, Object.assign({}, props, { name: "info" })) }),
            React.createElement(EdgeText, { style: [styles.message, passwordValid && styles.passed] }, lstrings.password_requirements)),
        list.map(({ title, validationStatus }) => {
            const color = validationStatus === 'met'
                ? styles.passed
                : validationStatus === 'error'
                    ? styles.error
                    : undefined;
            return (React.createElement(EdgeAnim, { enter: { type: 'fadeInDown', distance: 10 }, style: styles.passwordConditionRow, key: title },
                React.createElement(View, { style: styles.passwordConditionRow },
                    React.createElement(EdgeText, { style: [
                            styles.passwordConditionText,
                            styles.passwordConditionDot,
                            color
                        ] }, `\u2022`),
                    React.createElement(EdgeText, { style: [styles.passwordConditionText, color] }, title))));
        })));
};
const getStyles = cacheStyles((theme) => ({
    container: {
        backgroundColor: theme.cardBaseColor,
        borderRadius: theme.rem(1),
        padding: theme.rem(1),
        justifyContent: 'flex-start',
        margin: theme.rem(0.5)
    },
    error: {
        color: theme.dangerText
    },
    top: {
        flexDirection: 'row',
        alignItems: 'center',
        marginBottom: theme.rem(0.5)
    },
    message: {
        flexShrink: 1,
        fontFamily: theme.fontFaceBold,
        marginLeft: theme.rem(0.5)
    },
    passwordConditionRow: {
        marginLeft: theme.rem(0.4),
        flexDirection: 'row'
    },
    passwordConditionText: {
        flexShrink: 1,
        fontFamily: theme.fontFaceDefault,
        fontSize: theme.rem(0.75),
        marginBottom: theme.rem(0.15)
    },
    passwordConditionDot: {
        marginRight: theme.rem(0.5)
    },
    passed: {
        color: theme.positiveText
    },
    passedContainer: {
        borderColor: theme.positiveText
    }
}));
