var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import * as React from 'react';
import { useMemo } from 'react';
import { TextInput, View } from 'react-native';
import Animated, { interpolate, interpolateColor, useAnimatedRef, useAnimatedStyle, useDerivedValue, useSharedValue, withDelay, withTiming } from 'react-native-reanimated';
import { useHandler } from '../../hooks/useHandler';
import { useSpaceStyle } from '../../hooks/useSpaceStyle';
import { EdgeTouchableOpacity } from '../common/EdgeTouchableOpacity';
import { EdgeTouchableWithoutFeedback } from '../common/EdgeTouchableWithoutFeedback';
import { styled, styledWithRef } from '../hoc/styled';
import { CloseIconAnimated } from '../icons/ThemedIcons';
import { useTheme } from '../services/ThemeContext';
const AnimatedTextInput = Animated.createAnimatedComponent(TextInput);
export const SimpleTextInput = React.forwardRef((props, ref) => {
    const { 
    // Contents:
    placeholder, value, 
    // Appearance:
    iconComponent: Icon, scale: scaleProp, 
    // Callbacks:
    onBlur, onChangeText, onClear, onFocus, onSubmitEditing, 
    // TextInput:
    autoCapitalize, autoCorrect, autoFocus = false, blurOnClear = false, blurOnSubmit, disabled = false, inputAccessoryViewID, maxLength, secureTextEntry, testID } = props, spaceProps = __rest(props, ["placeholder", "value", "iconComponent", "scale", "onBlur", "onChangeText", "onClear", "onFocus", "onSubmitEditing", "autoCapitalize", "autoCorrect", "autoFocus", "blurOnClear", "blurOnSubmit", "disabled", "inputAccessoryViewID", "maxLength", "secureTextEntry", "testID"]);
    const theme = useTheme();
    const themeRem = theme.rem(1);
    const hasIcon = Icon != null;
    const hasValue = value !== '';
    // Imperative methods:
    const inputRef = useAnimatedRef();
    function blur() {
        if (inputRef.current != null)
            inputRef.current.blur();
    }
    function clear() {
        if (inputRef.current != null)
            inputRef.current.clear();
        if (onChangeText != null)
            onChangeText('');
        if (blurOnClear || !hasValue)
            blur();
        if (onClear != null)
            onClear();
    }
    function focus() {
        if (inputRef.current != null && !disabled)
            inputRef.current.focus();
    }
    function isFocused() {
        return inputRef.current != null ? inputRef.current.isFocused() : false;
    }
    function setNativeProps(nativeProps) {
        if (inputRef.current != null)
            inputRef.current.setNativeProps(nativeProps);
    }
    React.useImperativeHandle(ref, () => ({
        blur,
        clear,
        focus,
        isFocused,
        setNativeProps
    }));
    // Animates between 0 and 1 based our disabled state:
    const disableAnimation = useSharedValue(0);
    React.useEffect(() => {
        disableAnimation.value = withTiming(disabled ? 1 : 0);
    }, [disableAnimation, disabled]);
    // Animates between 0 and 1 based on focus:
    const baseDuration = 300;
    const focusAnimation = useSharedValue(0);
    // A delayed focus animation is required for closing the top border when
    // animating everything back to their original positions.
    const animationDelay = 0.4 * baseDuration;
    const handleBlur = useHandler(() => {
        focusAnimation.value = withDelay(animationDelay, withTiming(0, { duration: baseDuration }));
        if (onBlur != null)
            onBlur();
    });
    const handleClearPress = useHandler(() => {
        clear();
    });
    const handleFocus = useHandler(() => {
        focusAnimation.value = withTiming(1, { duration: baseDuration });
        if (onFocus != null)
            onFocus();
    });
    const handleSubmitEditing = useHandler(() => {
        if (onSubmitEditing != null)
            onSubmitEditing();
    });
    const leftIconSize = useDerivedValue(() => hasIcon
        ? hasValue
            ? 0
            : interpolate(focusAnimation.value, [0, 1], [themeRem, 0])
        : 0, [hasIcon, hasValue]);
    const rightIconSize = useDerivedValue(() => (hasValue ? themeRem : focusAnimation.value * themeRem), [hasValue]);
    const scale = useDerivedValue(() => { var _a; return (_a = scaleProp === null || scaleProp === void 0 ? void 0 : scaleProp.value) !== null && _a !== void 0 ? _a : 1; });
    const hasValueAnimation = useDerivedValue(() => hasValue
        ? withTiming(1, { duration: baseDuration })
        : withTiming(0, { duration: baseDuration }), [hasValue]);
    const interpolateIconColor = useAnimatedColorInterpolateFn(theme.textInputIconColor, theme.textInputIconColorFocused, theme.textInputIconColorDisabled);
    const iconColor = useDerivedValue(() => interpolateIconColor(focusAnimation, disableAnimation));
    return (React.createElement(EdgeTouchableWithoutFeedback, { accessible: false, testID: testID, onPress: () => focus() },
        React.createElement(Container, { disableAnimation: disableAnimation, focusAnimation: focusAnimation, scale: scale, spaceProps: spaceProps },
            React.createElement(SideContainer, { size: leftIconSize }, Icon == null ? null : React.createElement(Icon, { color: iconColor, size: leftIconSize })),
            React.createElement(InnerContainer, null,
                placeholder == null ? null : (React.createElement(PlaceholderText, { disableAnimation: disableAnimation, focusAnimation: focusAnimation, hasValueAnimation: hasValueAnimation, scale: scale }, placeholder)),
                React.createElement(InputField, { accessible: true, ref: inputRef, keyboardType: props.keyboardType, returnKeyType: props.returnKeyType, accessibilityState: { disabled }, autoFocus: autoFocus, disableAnimation: disableAnimation, focusAnimation: focusAnimation, selectionColor: theme.textInputTextColor, testID: `${testID}.textInput`, textAlignVertical: "top", scale: scale, value: value, 
                    // Callbacks:
                    onBlur: handleBlur, onChangeText: onChangeText, onFocus: handleFocus, onSubmitEditing: handleSubmitEditing, maxLength: maxLength, 
                    // Other Props:
                    autoCapitalize: autoCapitalize, autoCorrect: autoCorrect, blurOnSubmit: blurOnSubmit, inputAccessoryViewID: inputAccessoryViewID, secureTextEntry: secureTextEntry })),
            React.createElement(EdgeTouchableOpacity, { accessible: true, onPress: handleClearPress, testID: `${testID}.clearIcon` },
                React.createElement(SideContainer, { size: rightIconSize },
                    React.createElement(CloseIconAnimated, { color: iconColor, size: rightIconSize }))))));
});
const Container = styled(Animated.View)(theme => ({ disableAnimation, focusAnimation, scale, spaceProps }) => {
    const rem = theme.rem(1);
    const interpolateInputBackgroundColor = useAnimatedColorInterpolateFn(theme.textInputBackgroundColor, theme.textInputBackgroundColorFocused, theme.textInputBackgroundColorDisabled);
    const interpolateOutlineColor = useAnimatedColorInterpolateFn(theme.textInputBorderColor, theme.textInputBorderColorFocused, theme.textInputBorderColorDisabled);
    const spaceStyle = useSpaceStyle(spaceProps);
    return [
        spaceStyle,
        {
            alignItems: 'center',
            borderWidth: theme.textInputBorderWidth,
            borderRadius: theme.rem(theme.textInputBorderRadius),
            flexDirection: 'row',
            paddingHorizontal: theme.rem(1)
        },
        useAnimatedStyle(() => ({
            backgroundColor: interpolateInputBackgroundColor(focusAnimation, disableAnimation),
            borderColor: interpolateOutlineColor(focusAnimation, disableAnimation),
            opacity: interpolate(scale.value, [1, 0.5], [1, 0]),
            marginHorizontal: interpolate(scale.value, [1, 0], [0, 2 * rem]),
            paddingVertical: scale.value * 0.75 * rem
        }))
    ];
});
const SideContainer = styled(Animated.View)(theme => ({ size }) => {
    return [
        {
            alignItems: 'stretch',
            aspectRatio: 1
        },
        useAnimatedStyle(() => ({
            width: size.value,
            opacity: size.value
        }))
    ];
});
const InnerContainer = styled(View)({
    flex: 1
});
const PlaceholderText = styled(Animated.Text)(theme => ({ disableAnimation, focusAnimation, hasValueAnimation, scale }) => {
    const rem = theme.rem(1);
    const interpolatePlaceholderTextColor = useAnimatedColorInterpolateFn(theme.textInputPlaceholderColor, theme.textInputPlaceholderColorFocused, theme.textInputPlaceholderColorDisabled);
    return [
        {
            position: 'absolute',
            top: 0,
            alignItems: 'center',
            justifyContent: 'center',
            paddingHorizontal: theme.rem(0.5),
            paddingVertical: 0,
            margin: 0
        },
        {
            fontFamily: theme.fontFaceDefault,
            fontSize: theme.rem(1),
            includeFontPadding: false
        },
        useAnimatedStyle(() => {
            return {
                opacity: interpolate(hasValueAnimation.value, [0, 1], [1, 0]),
                color: interpolatePlaceholderTextColor(focusAnimation, disableAnimation),
                fontSize: scale.value * rem
            };
        })
    ];
});
const InputField = styledWithRef(AnimatedTextInput)(theme => ({ disableAnimation, focusAnimation, scale }) => {
    const rem = theme.rem(1);
    const interpolateTextColor = useAnimatedColorInterpolateFn(theme.textInputTextColor, theme.textInputTextColorFocused, theme.textInputTextColorDisabled);
    return [
        {
            flexGrow: 1,
            flexShrink: 1,
            fontFamily: theme.fontFaceDefault,
            paddingHorizontal: theme.rem(0.5),
            paddingVertical: 0,
            margin: 0
        },
        useAnimatedStyle(() => ({
            color: interpolateTextColor(focusAnimation, disableAnimation),
            fontSize: scale.value * rem
        }))
    ];
});
function useAnimatedColorInterpolateFn(fromColor, toColor, disabledColor) {
    const interpolateFn = useMemo(() => {
        return (focusValue, disabledValue) => {
            'worklet';
            const interFocusColor = interpolateColor(focusValue.value, [0, 1], [fromColor, toColor]);
            return interpolateColor(disabledValue.value, [0, 1], [interFocusColor, disabledColor]);
        };
    }, [fromColor, toColor, disabledColor]);
    return interpolateFn;
}
