/**
 * IMPORTANT: Changes in this file MUST be synced between edge-react-gui and
 * edge-login-ui-rn!
 */
import * as React from 'react';
import { Platform, View } from 'react-native';
import { EdgeAnim } from '../common/EdgeAnim';
import { maybeComponent } from '../hoc/maybeComponent';
import { styled } from '../hoc/styled';
import { Space } from '../layout/Space';
import { ButtonUi4 } from './ButtonUi4';
const INTER_BUTTON_SPACING_REM = 1;
const ANIM_DURATION = 1000;
const ANIM_DISTANCE_INCREMENT = 20;
/**
 * A consistently styled view for displaying button layouts.
 */
export const ButtonsViewUi4 = React.memo(({ absolute = false, primary, secondary, secondary2, tertiary, layout, parentType, animDistanceStart }) => {
    const buttonInfos = [primary, secondary, secondary2, tertiary].filter(key => key != null);
    if (layout == null) {
        layout = buttonInfos.length === 1 ? 'solo' : 'column';
    }
    const spacing = React.createElement(Space, { aroundRem: INTER_BUTTON_SPACING_REM / 2 });
    const renderButton = (type, buttonProps, index = 0) => {
        if (buttonProps == null)
            return null;
        const { label, onPress, disabled, spinner, testID } = buttonProps;
        const distance = animDistanceStart != null
            ? animDistanceStart + index * ANIM_DISTANCE_INCREMENT
            : undefined;
        const disableAnimation = Platform.OS === 'android';
        return (React.createElement(MaybeEdgeAnim, { when: animDistanceStart != null, disableAnimation: disableAnimation, enter: { type: 'fadeInDown', duration: ANIM_DURATION, distance } },
            React.createElement(ButtonUi4, { layout: layout, label: label, onPress: onPress, type: type, disabled: disabled, spinner: spinner, testID: testID })));
    };
    const hasPrimary = primary != null;
    const hasSecondary = secondary != null;
    const hasSecondary2 = secondary2 != null;
    const hasTertiary = tertiary != null;
    return (React.createElement(StyledButtonContainer, { absolute: absolute, layout: layout, parentType: parentType },
        hasPrimary && (React.createElement(React.Fragment, null,
            renderButton('primary', primary, 0),
            (hasSecondary || hasSecondary2 || hasTertiary) && spacing)),
        hasSecondary && (React.createElement(React.Fragment, null,
            renderButton('secondary', secondary, hasPrimary ? 1 : 0),
            (hasSecondary2 || hasTertiary) && spacing)),
        hasSecondary2 && (React.createElement(React.Fragment, null,
            renderButton('secondary', secondary2, (hasPrimary ? 1 : 0) + (hasSecondary ? 1 : 0)),
            hasTertiary && spacing)),
        hasTertiary && (React.createElement(React.Fragment, null, renderButton('tertiary', tertiary, (hasPrimary ? 1 : 0) +
            (hasSecondary ? 1 : 0) +
            (hasSecondary2 ? 1 : 0))))));
});
/** @deprecated - Shouldn't use this export post-UI4 transition once all our layouts have been codified into button layout components. */
export const StyledButtonContainer = styled(View)(theme => props => {
    const { absolute, layout, parentType } = props;
    const marginSize = theme.rem(0.5);
    const baseStyle = {
        margin: marginSize
    };
    const absoluteStyle = absolute
        ? {
            position: 'absolute',
            bottom: 0,
            left: marginSize,
            right: marginSize
        }
        : {};
    /** @deprecated Instead of a soloStyle case here, create a separate `SoloButton` component */
    const soloStyle = layout === 'solo'
        ? {
            justifyContent: 'center',
            marginHorizontal: theme.rem(0.5),
            alignItems: 'center',
            // TODO: This causes too much space in AddressModal.
            // Move flex to sceneMarginStyle, use in AddressModal, sync w/ LoginUi
            flexGrow: 1,
            flexShrink: 1
        }
        : {};
    const rowStyle = layout === 'row'
        ? {
            flexGrow: 1,
            flexShrink: 1,
            flexDirection: 'row-reverse',
            justifyContent: 'center'
        }
        : {};
    const columnStyle = layout === 'column'
        ? {
            alignSelf: 'center',
            alignItems: 'stretch',
            flexDirection: 'column'
        }
        : {};
    const sceneMarginStyle = parentType === 'scene'
        ? {
            flexGrow: 1,
            flexShrink: 0,
            justifyContent: 'flex-end',
            marginBottom: theme.rem(3),
            marginTop: theme.rem(1)
        }
        : {};
    const modalMarginStyle = parentType === 'modal'
        ? {
            marginBottom: theme.rem(1),
            marginTop: theme.rem(2)
        }
        : {};
    return Object.assign(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({}, baseStyle), absoluteStyle), soloStyle), rowStyle), columnStyle), sceneMarginStyle), modalMarginStyle);
});
const MaybeEdgeAnim = maybeComponent(EdgeAnim);
