import * as React from 'react';
const CACHE_MAX_SIZE = 10;
export function useFilter(allData, filterData = () => true) {
    const [filteredData, setFilteredData] = React.useState(allData);
    const [filteredDataCache, setFilteredDataCache] = React.useState({ '': allData });
    const [fifoCache, setFifoCache] = React.useState(['']);
    const setFilter = (filter) => {
        // If already existing in cache just return the existing sort
        if (filteredDataCache[filter] != null) {
            setFilteredData(filteredDataCache[filter]);
        }
        else {
            // Create the new filtered Array
            const newFilteredData = allData.filter((item, index) => filterData(filter, item, index));
            // Updated the cache
            filteredDataCache[filter] = newFilteredData;
            fifoCache.unshift(filter);
            // Check if over maximum cache size
            if (fifoCache.length >= CACHE_MAX_SIZE) {
                const cacheKey = fifoCache.pop();
                // @ts-expect-error
                // eslint-disable-next-line @typescript-eslint/no-dynamic-delete
                delete filteredDataCache[cacheKey];
            }
            // Update state
            setFifoCache(fifoCache);
            setFilteredDataCache(filteredDataCache);
            setFilteredData(newFilteredData);
        }
    };
    return [filteredData, setFilter];
}
