import { useRef } from 'react';
/**
 * A better version of `useCallback`.
 *
 * React itself may add this hook in a future version, called `useEvent`.
 * In the meantime, react-native-reanimated has a hook called `useEvent`,
 * so we don't want to conflict with that.
 */
export function useHandler(callback) {
    const bouncer = (...args) => stateRef.current.callback(...args);
    const stateRef = useRef({ callback, bouncer });
    stateRef.current.callback = callback;
    return stateRef.current.bouncer;
}
