declare const asGradientCoords: import("cleaners").ObjectCleaner<{
    x: number;
    y: number;
}>;
type GradientCoords = ReturnType<typeof asGradientCoords>;
declare const asThemeGradientParams: import("cleaners").ObjectCleaner<{
    colors: string[];
    start: {
        x: any;
        y: any;
    };
    end: {
        x: any;
        y: any;
    };
}>;
type ThemeGradientParams = ReturnType<typeof asThemeGradientParams>;
declare const asThemeShadowParams: import("cleaners").ObjectCleaner<{
    shadowColor: string;
    shadowOffset: {
        width: any;
        height: any;
    };
    shadowOpacity: number;
    shadowRadius: number;
    elevation: number;
}>;
type ThemeShadowParams = ReturnType<typeof asThemeShadowParams>;
declare const asTextShadowParams: import("cleaners").ObjectCleaner<{
    textShadowColor: string;
    textShadowOffset: {
        width: any;
        height: any;
    };
    textShadowRadius: number;
}>;
type TextShadowParams = ReturnType<typeof asTextShadowParams>;
export declare const themeNoShadow: ThemeShadowParams;
export declare const textNoShadow: TextShadowParams;
export declare const asOptionalTheme: import("cleaners").ObjectCleaner<Partial<Theme>>;
export type OptionalTheme = ReturnType<typeof asOptionalTheme>;
type FontWeight = 'normal' | 'bold' | '100' | '200' | '300' | '400' | '500' | '600' | '700' | '800' | '900';
export interface Theme {
    rem: (size: number) => number;
    preferPrimaryButton: boolean;
    fontFamily: string;
    fontWeightBold: FontWeight;
    isDark: boolean;
    icon: string;
    iconDeactivated: string;
    iconTappable: string;
    modal: string;
    modalCloseIcon: string;
    modalBorderColor: string;
    modalBorderWidth: number;
    modalBorderRadiusRem: number;
    modalBackground: string;
    modalSceneOverlayColor: string;
    modalDragbarColor: string;
    primaryText: string;
    secondaryText: string;
    dangerText: string;
    warningText: string;
    linkText: string;
    positiveText: string;
    deactivatedText: string;
    tileBackground: string;
    buttonBorderRadiusRem: number;
    keypadButtonOutline: string;
    keypadButtonOutlineWidth: number;
    keypadButton: string[];
    keypadButtonColorStart: GradientCoords;
    keypadButtonColorEnd: GradientCoords;
    keypadButtonText: string;
    keypadButtonTextShadow: TextShadowParams;
    keypadButtonShadow: ThemeShadowParams;
    keypadButtonBorderRadiusRem: number;
    keypadButtonFontSizeRem: number;
    keypadButtonFont: string;
    primaryButtonOutline: string;
    primaryButtonOutlineWidth: number;
    primaryButton: string[];
    primaryButtonColorStart: GradientCoords;
    primaryButtonColorEnd: GradientCoords;
    primaryButtonText: string;
    primaryButtonTextShadow: TextShadowParams;
    primaryButtonShadow: ThemeShadowParams;
    primaryButtonFontSizeRem: number;
    primaryButtonFont: string;
    secondaryButtonOutline: string;
    secondaryButtonOutlineWidth: number;
    secondaryButton: string[];
    secondaryButtonColorStart: GradientCoords;
    secondaryButtonColorEnd: GradientCoords;
    secondaryButtonText: string;
    secondaryButtonTextShadow: TextShadowParams;
    secondaryButtonShadow: ThemeShadowParams;
    secondaryButtonFontSizeRem: number;
    secondaryButtonFont: string;
    escapeButtonOutline: string;
    escapeButtonOutlineWidth: number;
    escapeButton: string[];
    escapeButtonColorStart: GradientCoords;
    escapeButtonColorEnd: GradientCoords;
    escapeButtonText: string;
    escapeButtonTextShadow: TextShadowParams;
    escapeButtonShadow: ThemeShadowParams;
    escapeButtonFontSizeRem: number;
    escapeButtonFont: string;
    pinUsernameButtonOutline: string;
    pinUsernameButtonOutlineWidth: number;
    pinUsernameButton: string[];
    pinUsernameButtonColorStart: GradientCoords;
    pinUsernameButtonColorEnd: GradientCoords;
    pinUsernameButtonText: string;
    pinUsernameButtonTextShadow: TextShadowParams;
    pinUsernameButtonShadow: ThemeShadowParams;
    pinUsernameButtonBorderRadiusRem: number;
    pinUsernameButtonFontSizeRem: number;
    pinUsernameButtonFont: string;
    outlineTextInputColor: string;
    outlineTextInputTextColor: string;
    outlineTextInputBorderWidth: number;
    outlineTextInputBorderColor: string;
    outlineTextInputBorderColorFocused: string;
    outlineTextInputLabelColor: string;
    outlineTextInputLabelColorFocused: string;
    textInputTextColor: string;
    textInputTextColorDisabled: string;
    textInputTextColorFocused: string;
    textInputBackgroundColor: string;
    textInputBackgroundColorDisabled: string;
    textInputBackgroundColorFocused: string;
    textInputBorderWidth: number;
    textInputBorderColor: string;
    textInputBorderColorDisabled: string;
    textInputBorderColorFocused: string;
    textInputBorderRadius: number;
    textInputIconColor: string;
    textInputIconColorDisabled: string;
    textInputIconColorFocused: string;
    textInputPlaceholderColor: string;
    textInputPlaceholderColorDisabled: string;
    textInputPlaceholderColorFocused: string;
    dropdownWarning: string;
    dropdownError: string;
    dropdownText: string;
    dateModalTextLight: string;
    dateModalTextDark: string;
    dateModalBackgroundLight: string;
    dateModalBackgroundDark: string;
    cardBorder: number;
    cardBorderColor: string;
    cardBorderRadius: number;
    lineDivider: string;
    thinLineWidth: number;
    mediumLineWidth: number;
    fontFaceDefault: string;
    fontFaceMedium: string;
    fontFaceBold: string;
    fontFaceSymbols: string;
    cardBaseColor: string;
    cardGradientWarning: ThemeGradientParams;
    cardGradientError: ThemeGradientParams;
    cardOverlayDisabled: string;
}
export {};
