var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { asBoolean, asJSON, asMaybe, asObject } from 'cleaners';
import { makeReactNativeDisklet } from 'disklet';
import * as React from 'react';
import { NativeModules, Platform } from 'react-native';
import { checkNotifications, openSettings, requestNotifications, RESULTS } from 'react-native-permissions';
import { sprintf } from 'sprintf-js';
import { lstrings } from '../common/locales/strings';
import { RequestPermissionsModal } from '../components/modals/RequestPermissionsModal';
import { Airship } from '../components/services/AirshipInstance';
const { AbcCoreJsUi } = NativeModules;
const disklet = makeReactNativeDisklet();
const notificationPermissionsInfoFile = 'notificationsPermisions.json';
const isIos = Platform.OS === 'ios';
//
// Logic for showing notification/app refresh request
//
// 0 = false, 1 = true
// nr = notifications and app refresh request message
// n = notifications request message
// r = app refresh request only message
//
// | Notif Enabled |  Notif Block | App Refresh Enabled || Message |
// |---------------|--------------|---------------------||---------|
// |       0       |       0      |           0         >>    nr   |
// |       0       |       0      |           1         >>    n    |
// |       0       |       1      |           0         >>    r    |
// |       0       |       1      |           1         >>         |
// |       1       |       0      |           0         >>    r    |
// |       1       |       0      |           1         >>         |
// |       1       |       1      |           0         >>    r    |
// |       1       |       1      |           1         >>         |
//
const logicMap = [
    [[], []],
    [[], []]
];
logicMap[0][0][0] = lstrings.notifications_and_refresh_permissions_branded_s;
logicMap[0][0][1] = lstrings.notifications_permissions_branded_s;
logicMap[0][1][0] = lstrings.refresh_permission_branded_s;
logicMap[0][1][1] = undefined;
logicMap[1][0][0] = lstrings.refresh_permission_branded_s;
logicMap[1][0][1] = undefined;
logicMap[1][1][0] = lstrings.refresh_permission_branded_s;
logicMap[1][1][1] = undefined;
/**
 * Asks the users for notification permissions, if needed. Returns true if the
 * permissions modal was shown, false if not.
 */
export const showNotificationPermissionReminder = (opts) => __awaiter(void 0, void 0, void 0, function* () {
    var _a, _b, _c;
    const { appName = lstrings.app_name_default, onLogEvent = () => { }, onNotificationPermit } = opts;
    const notificationPermission = yield checkNotifications();
    const notificationStatus = notificationPermission.status;
    const notifEnabled = notificationStatus !== RESULTS.BLOCKED &&
        notificationStatus !== RESULTS.DENIED
        ? 1
        : 0;
    const statusAppRefresh = isIos
        ? yield AbcCoreJsUi.backgroundAppRefreshStatus().catch((error) => console.log(error))
        : undefined;
    const refreshEnabled = statusAppRefresh !== RESULTS.BLOCKED ? 1 : 0;
    const notificationPermissionsInfoJson = yield disklet
        .getText(notificationPermissionsInfoFile)
        .catch(error => console.log(error));
    const notificationPermissionsInfo = asMaybe(asNotificationPermissionsInfo)(notificationPermissionsInfoJson);
    const isNotificationBlocked = notificationPermissionsInfo
        ? notificationPermissionsInfo.isNotificationBlocked
        : false;
    const isNotificationBlockedBit = isNotificationBlocked ? 1 : 0;
    const permissionMessage = logicMap[notifEnabled][isNotificationBlockedBit][refreshEnabled] != null
        ? sprintf((_a = logicMap[notifEnabled][isNotificationBlockedBit][refreshEnabled]) !== null && _a !== void 0 ? _a : '', appName)
        : undefined;
    console.log(`checkAndRequestNotifications`);
    console.log(`notificationStatus:${notificationStatus} statusAppRefresh:${statusAppRefresh}`);
    console.log(`notifEnabled:${notifEnabled} notifBlocked:${isNotificationBlockedBit} refreshEnabled:${refreshEnabled}`);
    console.log(`permissionMessage:${permissionMessage}`);
    if (permissionMessage == null) {
        if (onNotificationPermit != null && notificationPermissionsInfo != null) {
            onNotificationPermit(notificationPermissionsInfo);
        }
        return false;
    }
    const choiceMap = {
        optInPriceChanges: !((_b = notificationPermissionsInfo === null || notificationPermissionsInfo === void 0 ? void 0 : notificationPermissionsInfo.notificationOptIns.ignorePriceChanges) !== null && _b !== void 0 ? _b : false),
        optInMarketing: !((_c = notificationPermissionsInfo === null || notificationPermissionsInfo === void 0 ? void 0 : notificationPermissionsInfo.notificationOptIns.ignoreMarketing) !== null && _c !== void 0 ? _c : false)
    };
    const choices = yield Airship.show(bridge => (React.createElement(RequestPermissionsModal, { bridge: bridge, message: permissionMessage, choices: choiceMap })));
    console.log(`checkAndRequestNotifications result ${JSON.stringify(choices)}`);
    // Don't do anything if the user didn't make a choice (dismissed modal)
    if (choices == null)
        return true;
    const newPermissionsInfo = {
        isNotificationBlocked: !(choices === null || choices === void 0 ? void 0 : choices.enable),
        notificationOptIns: {
            ignoreMarketing: !choices.optInMarketing,
            ignorePriceChanges: !choices.optInPriceChanges
        }
    };
    yield disklet.setText(notificationPermissionsInfoFile, JSON.stringify(newPermissionsInfo));
    if (choices.enable) {
        if (notificationStatus === RESULTS.DENIED) {
            requestNotifications(isIos ? ['alert', 'badge', 'sound'] : []).catch(error => console.log(error));
        }
        if (notificationStatus === RESULTS.BLOCKED ||
            statusAppRefresh === RESULTS.BLOCKED) {
            openSettings().catch(error => console.log(error));
        }
        // Only log enable, not deny/block due to our privacy policy.
        onLogEvent('Permission_Modal_Notification_Enable');
    }
    if (onNotificationPermit != null)
        onNotificationPermit(newPermissionsInfo);
    return true;
});
const asNotificationPermissionsInfo = asJSON(asObject({
    isNotificationBlocked: asBoolean,
    notificationOptIns: asObject({
        ignoreMarketing: asBoolean,
        ignorePriceChanges: asBoolean
    })
}));
