var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as React from 'react';
import { Keyboard } from 'react-native';
import { sprintf } from 'sprintf-js';
import { lstrings } from '../common/locales/strings';
import { ButtonsModal } from '../components/modals/ButtonsModal';
import { Airship } from '../components/services/AirshipInstance';
const OTP_REMINDER_MILLISECONDS = 7 * 24 * 60 * 60 * 1000;
const OTP_REMINDER_STORE_NAME = 'app.edge.login';
const OTP_REMINDER_KEY_NAME_LAST_OTP_CHECKED = 'lastOtpCheck';
const OTP_REMINDER_KEY_NAME_DONT_ASK = 'OtpDontAsk';
/**
 * Check and show the 2fa reminder on login, if necessary.
 */
export function showOtpReminder(account) {
    return __awaiter(this, void 0, void 0, function* () {
        const { otpKey, dataStore, username, created } = account;
        if (username == null)
            return;
        if (otpKey != null)
            return;
        const [dontAsk, lastOtpCheckString] = (yield Promise.all([
            dataStore
                .getItem(OTP_REMINDER_STORE_NAME, OTP_REMINDER_KEY_NAME_DONT_ASK)
                .catch(() => null),
            dataStore
                .getItem(OTP_REMINDER_STORE_NAME, OTP_REMINDER_KEY_NAME_LAST_OTP_CHECKED)
                .catch(() => null)
        ]));
        if (dontAsk)
            return;
        const lastOtpCheck = lastOtpCheckString != null ? parseInt(lastOtpCheckString) : null;
        // Show the modal if we have never shown it before,
        // and the account is old enough:
        if (lastOtpCheck == null &&
            (created == null ||
                Date.now() > created.valueOf() + OTP_REMINDER_MILLISECONDS)) {
            Keyboard.dismiss();
            const result = yield Airship.show(bridge => (React.createElement(ButtonsModal, { bridge: bridge, title: lstrings.otp_reset_modal_header, message: lstrings.otp_reset_modal_message, buttons: {
                    yes: { label: lstrings.enable },
                    no: { label: lstrings.skip_button, type: 'secondary' }
                } })));
            if (result === 'yes') {
                yield enableOtp(account);
            }
            else {
                yield account.dataStore.setItem(OTP_REMINDER_STORE_NAME, OTP_REMINDER_KEY_NAME_LAST_OTP_CHECKED, Date.now().toString());
            }
        }
        // Show a modal with the "Don't ask again" button if we have waited
        // long enough since the last time we showed a 2fa reminder modal:
        if (lastOtpCheck != null &&
            Date.now() > lastOtpCheck + OTP_REMINDER_MILLISECONDS) {
            Keyboard.dismiss();
            const result = yield Airship.show(bridge => (React.createElement(ButtonsModal, { bridge: bridge, title: lstrings.otp_reset_modal_header, message: lstrings.otp_reset_modal_message, buttons: {
                    enable: { label: lstrings.enable, type: 'primary' },
                    cancel: { label: lstrings.skip_button, type: 'secondary' },
                    dontAsk: {
                        label: lstrings.otp_reset_modal_dont_ask,
                        type: 'secondary'
                    }
                } })));
            if (result === 'enable') {
                yield enableOtp(account);
            }
            else if (result === 'dontAsk') {
                yield account.dataStore.setItem(OTP_REMINDER_STORE_NAME, OTP_REMINDER_KEY_NAME_DONT_ASK, 'true');
            }
            else {
                yield account.dataStore.setItem(OTP_REMINDER_STORE_NAME, OTP_REMINDER_KEY_NAME_LAST_OTP_CHECKED, Date.now().toString());
            }
        }
    });
}
const enableOtp = (account) => __awaiter(void 0, void 0, void 0, function* () {
    yield account.enableOtp();
    return yield Airship.show(bridge => (React.createElement(ButtonsModal, { bridge: bridge, title: lstrings.otp_authentication_header, message: sprintf(lstrings.otp_authentication_message, account.otpKey), buttons: { ok: { label: lstrings.ok } } })));
});
